@@ -1,5 +1,5 @@
 # engine/reflection.py
-# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -157,7 +157,7 @@ class Inspector(object):
         """Return all table names in referred to within a particular schema.
 
         The names are expected to be real tables only, not views.
-        Views are instead returned using the :meth:`.get_view_names`
+        Views are instead returned using the :meth:`.Inspector.get_view_names`
         method.
 
 
@@ -311,9 +311,6 @@ class Inspector(object):
         name
           optional name of the foreign key constraint.
 
-        \**kw
-          other options passed to the dialect's get_foreign_keys() method.
-
         """
 
         return self.dialect.get_foreign_keys(self.bind, table_name, schema,
@@ -335,14 +332,31 @@ class Inspector(object):
         unique
           boolean
 
-        \**kw
-          other options passed to the dialect's get_indexes() method.
         """
 
         return self.dialect.get_indexes(self.bind, table_name,
                                                   schema,
                                             info_cache=self.info_cache, **kw)
 
+    def get_unique_constraints(self, table_name, schema=None, **kw):
+        """Return information about unique constraints in `table_name`.
+
+        Given a string `table_name` and an optional string `schema`, return
+        unique constraint information as a list of dicts with these keys:
+
+        name
+          the unique constraint's name
+
+        column_names
+          list of column names in order
+
+        .. versionadded:: 0.8.4
+
+        """
+
+        return self.dialect.get_unique_constraints(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw)
+
     def reflecttable(self, table, include_columns, exclude_columns=()):
         """Given a Table object, load its internal constructs based on
         introspection.
