@@ -1,16 +1,15 @@
 # orm/identity.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 import weakref
-from sqlalchemy.orm import attributes
+from . import attributes
 
 
 class IdentityMap(dict):
     def __init__(self):
-        self._mutable_attrs = set()
         self._modified = set()
         self._wr = weakref.ref(self)
 
@@ -20,46 +19,31 @@ class IdentityMap(dict):
     def add(self, state):
         raise NotImplementedError()
 
-    def discard(self, state):
-        raise NotImplementedError()
-
     def update(self, dict):
         raise NotImplementedError("IdentityMap uses add() to insert data")
 
     def clear(self):
         raise NotImplementedError("IdentityMap uses remove() to remove data")
 
-    def remove(self, state):
-        # deprecated, here for 0.6 compat
-        return self.discard(state)
-
     def _manage_incoming_state(self, state):
         state._instance_dict = self._wr
 
         if state.modified:
             self._modified.add(state)
-        if state.manager.mutable_attributes:
-            self._mutable_attrs.add(state)
 
     def _manage_removed_state(self, state):
         del state._instance_dict
-        self._mutable_attrs.discard(state)
         self._modified.discard(state)
 
     def _dirty_states(self):
-        return self._modified.union(s for s in self._mutable_attrs.copy()
-                                    if s.modified)
+        return self._modified
 
     def check_modified(self):
-        """return True if any InstanceStates present have been marked as 'modified'."""
+        """return True if any InstanceStates present have been marked
+        as 'modified'.
 
-        if self._modified:
-            return True
-        else:
-            for state in self._mutable_attrs.copy():
-                if state.modified:
-                    return True
-        return False
+        """
+        return bool(self._modified)
 
     def has_key(self, key):
         return key in self
@@ -82,6 +66,7 @@ class IdentityMap(dict):
     def __delitem__(self, key):
         raise NotImplementedError("IdentityMap uses remove() to remove data")
 
+
 class WeakInstanceDict(IdentityMap):
     def __init__(self):
         IdentityMap.__init__(self)
@@ -89,8 +74,6 @@ class WeakInstanceDict(IdentityMap):
     def __getitem__(self, key):
         state = dict.__getitem__(self, key)
         o = state.obj()
-        if o is None:
-            o = state._is_really_none()
         if o is None:
             raise KeyError, key
         return o
@@ -100,8 +83,6 @@ class WeakInstanceDict(IdentityMap):
             if dict.__contains__(self, key):
                 state = dict.__getitem__(self, key)
                 o = state.obj()
-                if o is None:
-                    o = state._is_really_none()
             else:
                 return False
         except KeyError:
@@ -131,12 +112,11 @@ class WeakInstanceDict(IdentityMap):
                 existing_state = dict.__getitem__(self, key)
                 if existing_state is not state:
                     o = existing_state.obj()
-                    if o is None:
-                        o = existing_state._is_really_none()
                     if o is not None:
-                        raise AssertionError("A conflicting state is already "
-                                        "present in the identity map for key %r" 
-                                        % (key, ))
+                        raise AssertionError(
+                            "A conflicting state is already "
+                            "present in the identity map for key %r"
+                            % (key, ))
                 else:
                     return
             except KeyError:
@@ -150,9 +130,7 @@ class WeakInstanceDict(IdentityMap):
             return default
         o = state.obj()
         if o is None:
-            o = state._is_really_none()
-            if o is None:
-                return default
+            return default
         return o
 
     def _items(self):
@@ -182,10 +160,12 @@ class WeakInstanceDict(IdentityMap):
     #    return iter(self._values())
     # Py2K
     items = _items
+
     def iteritems(self):
         return iter(self.items())
 
     values = _values
+
     def itervalues(self):
         return iter(self.values())
     # end Py2K
@@ -200,18 +180,21 @@ class WeakInstanceDict(IdentityMap):
     def discard(self, state):
         st = dict.get(self, state.key, None)
         if st is state:
-            dict.__delitem__(self, state.key)
+            dict.pop(self, state.key, None)
             self._manage_removed_state(state)
 
     def prune(self):
         return 0
 
+
 class StrongInstanceDict(IdentityMap):
     def all_states(self):
         return [attributes.instance_state(o) for o in self.itervalues()]
 
     def contains_state(self, state):
-        return state.key in self and attributes.instance_state(self[state.key]) is state
+        return (
+            state.key in self and
+            attributes.instance_state(self[state.key]) is state)
 
     def replace(self, state):
         if dict.__contains__(self, state.key):
@@ -241,7 +224,7 @@ class StrongInstanceDict(IdentityMap):
         if obj is not None:
             st = attributes.instance_state(obj)
             if st is state:
-                dict.__delitem__(self, state.key)
+                dict.pop(self, state.key, None)
                 self._manage_removed_state(state)
 
     def prune(self):
@@ -258,4 +241,3 @@ class StrongInstanceDict(IdentityMap):
         dict.update(self, keepers)
         self.modified = bool(dirty)
         return ref_count - len(self)
-
