@@ -895,7 +895,7 @@ class SubqueryLoader(AbstractRelationshipLoader):
                     attr = getattr(parent_alias, key).\
                                     of_type(effective_entity)
                 else:
-                    attr = key
+                    attr = getattr(mapper.entity, key)
 
             if second_to_last:
                 q = q.join(parent_alias, attr, from_joinpoint=True)
@@ -1043,7 +1043,7 @@ class JoinedLoader(AbstractRelationshipLoader):
                                 column_collection=None, parentmapper=None,
                                 allow_innerjoin=True,
                                 **kwargs):
-        """Add a left outer join to the statement thats being constructed."""
+        """Add a left outer join to the statement that's being constructed."""
 
         if not context.query._enable_eagerloads:
             return
@@ -1270,7 +1270,7 @@ class JoinedLoader(AbstractRelationshipLoader):
             self.mapper.identity_key_from_row(row, decorator)
             return decorator
         except KeyError:
-            # no identity key - dont return a row
+            # no identity key - don't return a row
             # processor, will cause a degrade to lazy
             return False
 
