@@ -123,7 +123,7 @@ def polymorphic_union(table_map, typecolname,
     for key in table_map.keys():
         table = table_map[key]
 
-        # mysql doesnt like selecting from a select;
+        # mysql doesn't like selecting from a select;
         # make it an alias of the select
         if isinstance(table, sql.Select):
             table = table.alias()
@@ -1075,13 +1075,13 @@ def _inspect_mapped_class(class_, configure=False):
         if not class_manager.is_mapped:
             return None
         mapper = class_manager.mapper
+    except exc.NO_STATE:
+        return None
+    else:
         if configure and mapperlib.module._new_mappers:
             mapperlib.configure_mappers()
         return mapper
 
-    except exc.NO_STATE:
-        return None
-
 
 def object_mapper(instance):
     """Given an object, return the primary Mapper associated with the object
