@@ -395,7 +395,7 @@ class TimezoneTest(fixtures.TestBase):
 
     psycopg will return a datetime with a tzinfo attached to it, if
     postgresql returns it.  python then will not let you compare a
-    datetime with a tzinfo to a datetime that doesnt have one.  this
+    datetime with a tzinfo to a datetime that doesn't have one.  this
     test illustrates two ways to have datetime types with and without
     timezone info. """
 
@@ -1400,6 +1400,21 @@ class HStoreRoundTripTest(fixtures.TablesTest):
         )
         self._assert_data([{r'key \"foo\"': r'value \"bar"\ xyz'}])
 
+    def test_orm_round_trip(self):
+        from sqlalchemy import orm
+        class Data(object):
+            def __init__(self, name, data):
+                self.name = name
+                self.data = data
+        orm.mapper(Data, self.tables.data_table)
+        s = orm.Session(testing.db)
+        d = Data(name='r1', data={"key1": "value1", "key2": "value2",
+                                    "key3": "value3"})
+        s.add(d)
+        eq_(
+            s.query(Data.data, Data).all(),
+            [(d.data, d)]
+        )
 class _RangeTypeMixin(object):
     __requires__ = 'range_types',
     __dialect__ = 'postgresql+psycopg2'
