@@ -1,22 +1,30 @@
-from test.lib.testing import eq_, assert_raises, assert_raises_message, config
+# coding: utf-8
+from __future__ import with_statement
+
+from sqlalchemy.testing import eq_, assert_raises, assert_raises_message, \
+    config, is_
 import re
-from test.lib.util import picklers
+from sqlalchemy.testing.util import picklers
 from sqlalchemy.interfaces import ConnectionProxy
 from sqlalchemy import MetaData, Integer, String, INT, VARCHAR, func, \
     bindparam, select, event, TypeDecorator, create_engine, Sequence
 from sqlalchemy.sql import column, literal
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 import sqlalchemy as tsa
-from test.lib import testing, engines
-from test.lib.engines import testing_engine
-import logging
+from sqlalchemy import testing
+from sqlalchemy import util
+from sqlalchemy.testing import engines
+from sqlalchemy.testing.engines import testing_engine
+import logging.handlers
 from sqlalchemy.dialects.oracle.zxjdbc import ReturningParam
-from sqlalchemy.engine import base, default
+from sqlalchemy.engine import result as _result, default
 from sqlalchemy.engine.base import Connection, Engine
-from test.lib import fixtures
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.mock import Mock, call
 import StringIO
 
-users, metadata = None, None
+
+users, metadata, users_autoinc = None, None, None
 class ExecuteTest(fixtures.TestBase):
     @classmethod
     def setup_class(cls):
@@ -41,12 +49,12 @@ class ExecuteTest(fixtures.TestBase):
     def teardown_class(cls):
         metadata.drop_all()
 
-    @testing.fails_on("postgresql+pg8000", 
+    @testing.fails_on("postgresql+pg8000",
             "pg8000 still doesn't allow single % without params")
     def test_no_params_option(self):
-        stmt = "SELECT '%'"
-        if testing.against('oracle'):
-            stmt += " FROM DUAL"
+        stmt = "SELECT '%'" + testing.db.dialect.statement_compiler(
+                                    testing.db.dialect, None).default_from()
+
         conn = testing.db.connect()
         result = conn.\
                 execution_options(no_parameters=True).\
@@ -79,6 +87,22 @@ class ExecuteTest(fixtures.TestBase):
                 (6, 'donkey'),
                 (7, 'sally'),
                 ]
+            for multiparam, param in [
+                (("jack", "fred"), {}),
+                ((["jack", "fred"],), {})
+            ]:
+                res = conn.execute(
+                    "select * from users where user_name=? or "
+                    "user_name=? order by user_id",
+                    *multiparam, **param)
+                assert res.fetchall() == [
+                    (1, 'jack'),
+                    (2, 'fred')
+                ]
+            res = conn.execute("select * from users where user_name=?",
+                "jack"
+            )
+            assert res.fetchall() == [(1, 'jack')]
             conn.execute('delete from users')
 
         go(testing.db)
@@ -90,7 +114,7 @@ class ExecuteTest(fixtures.TestBase):
 
     # some psycopg2 versions bomb this.
     @testing.fails_on_everything_except('mysql+mysqldb', 'mysql+pymysql',
-            'mysql+mysqlconnector', 'postgresql')
+            'mysql+cymysql', 'mysql+mysqlconnector', 'postgresql')
     @testing.fails_on('postgresql+zxjdbc', 'sprintf not supported')
     def test_raw_sprintf(self):
         def go(conn):
@@ -104,6 +128,23 @@ class ExecuteTest(fixtures.TestBase):
             res = conn.execute('select * from users order by user_id')
             assert res.fetchall() == [(1, 'jack'), (2, 'ed'), (3,
                     'horse'), (4, 'sally'), (5, None)]
+            for multiparam, param in [
+                (("jack", "ed"), {}),
+                ((["jack", "ed"],), {})
+            ]:
+                res = conn.execute(
+                    "select * from users where user_name=%s or "
+                    "user_name=%s order by user_id",
+                    *multiparam, **param)
+                assert res.fetchall() == [
+                    (1, 'jack'),
+                    (2, 'ed')
+                ]
+            res = conn.execute("select * from users where user_name=%s",
+                "jack"
+            )
+            assert res.fetchall() == [(1, 'jack')]
+
             conn.execute('delete from users')
         go(testing.db)
         conn = testing.db.connect()
@@ -119,8 +160,8 @@ class ExecuteTest(fixtures.TestBase):
     @testing.skip_if(lambda : testing.against('mysql+mysqldb'),
                      'db-api flaky')
     @testing.fails_on_everything_except('postgresql+psycopg2',
-            'postgresql+pypostgresql', 'mysql+mysqlconnector', 
-            'mysql+pymysql')
+            'postgresql+pypostgresql', 'mysql+mysqlconnector',
+            'mysql+pymysql', 'mysql+cymysql')
     def test_raw_python(self):
         def go(conn):
             conn.execute('insert into users (user_id, user_name) '
@@ -165,19 +206,37 @@ class ExecuteTest(fixtures.TestBase):
         finally:
             conn.close()
 
+    @testing.engines.close_open_connections
     def test_exception_wrapping_dbapi(self):
-        def go(conn):
+        conn = testing.db.connect()
+        for _c in testing.db, conn:
             assert_raises_message(
                 tsa.exc.DBAPIError,
                 r"not_a_valid_statement",
-                conn.execute, 'not_a_valid_statement'
+                _c.execute, 'not_a_valid_statement'
             )
-        go(testing.db)
-        conn = testing.db.connect()
-        try:
-            go(conn)
-        finally:
-            conn.close()
+
+    @testing.requires.sqlite
+    def test_exception_wrapping_non_dbapi_error(self):
+        e = create_engine('sqlite://')
+        e.dialect.is_disconnect = is_disconnect = Mock()
+
+        c = e.connect()
+
+        c.connection.cursor = Mock(
+                return_value=Mock(
+                    execute=Mock(
+                            side_effect=TypeError("I'm not a DBAPI error")
+                    ))
+                )
+
+        assert_raises_message(
+            TypeError,
+            "I'm not a DBAPI error",
+            c.execute, "select "
+        )
+        eq_(is_disconnect.call_count, 0)
+
 
     def test_exception_wrapping_non_dbapi_statement(self):
         class MyType(TypeDecorator):
@@ -188,7 +247,7 @@ class ExecuteTest(fixtures.TestBase):
         def _go(conn):
             assert_raises_message(
                 tsa.exc.StatementError,
-                r"nope \(original cause: Exception: nope\) 'SELECT 1 ",
+                r"nope \(original cause: Exception: nope\) u?'SELECT 1 ",
                 conn.execute,
                     select([1]).\
                         where(
@@ -202,14 +261,31 @@ class ExecuteTest(fixtures.TestBase):
         finally:
             conn.close()
 
+    def test_stmt_exception_non_ascii(self):
+        name = util.u('méil')
+        assert_raises_message(
+            tsa.exc.StatementError,
+            util.u(
+                "A value is required for bind parameter 'uname'"
+                r'.*SELECT users.user_name AS .m\\xe9il.') if util.py2k
+            else
+                util.u(
+                    "A value is required for bind parameter 'uname'"
+                    '.*SELECT users.user_name AS .méil.'),
+            testing.db.execute,
+            select([users.c.user_name.label(name)]).where(
+                            users.c.user_name == bindparam("uname")),
+            {'uname_incorrect': 'foo'}
+        )
+
     def test_stmt_exception_pickleable_no_dbapi(self):
         self._test_stmt_exception_pickleable(Exception("hello world"))
 
-    @testing.fails_on("postgresql+psycopg2", 
-                "Packages the cursor in the exception")
-    @testing.fails_on("mysql+oursql", 
+    @testing.crashes("postgresql+psycopg2",
+                "Older versions don't support cursor pickling, newer ones do")
+    @testing.fails_on("mysql+oursql",
                 "Exception doesn't come back exactly the same from pickle")
-    @testing.fails_on("oracle+cx_oracle", 
+    @testing.fails_on("oracle+cx_oracle",
                         "cx_oracle exception seems to be having "
                         "some issue with pickling")
     def test_stmt_exception_pickleable_plus_dbapi(self):
@@ -228,12 +304,12 @@ class ExecuteTest(fixtures.TestBase):
 
     def _test_stmt_exception_pickleable(self, orig):
         for sa_exc in (
-            tsa.exc.StatementError("some error", 
-                            "select * from table", 
-                           {"foo":"bar"}, 
+            tsa.exc.StatementError("some error",
+                            "select * from table",
+                           {"foo":"bar"},
                             orig),
-            tsa.exc.InterfaceError("select * from table", 
-                            {"foo":"bar"}, 
+            tsa.exc.InterfaceError("select * from table",
+                            {"foo":"bar"},
                             orig),
             tsa.exc.NoReferencedTableError("message", "tname"),
             tsa.exc.NoReferencedColumnError("message", "tname", "cname"),
@@ -246,7 +322,7 @@ class ExecuteTest(fixtures.TestBase):
                     eq_(repickled.params, {"foo":"bar"})
                     eq_(repickled.statement, sa_exc.statement)
                     if hasattr(sa_exc, "connection_invalidated"):
-                        eq_(repickled.connection_invalidated, 
+                        eq_(repickled.connection_invalidated,
                             sa_exc.connection_invalidated)
                     eq_(repickled.orig.args[0], orig.args[0])
 
@@ -279,16 +355,14 @@ class ExecuteTest(fixtures.TestBase):
     def test_empty_insert(self):
         """test that execute() interprets [] as a list with no params"""
 
-        result = \
-            testing.db.execute(users_autoinc.insert().
-                        values(user_name=bindparam('name')), [])
-        eq_(testing.db.execute(users_autoinc.select()).fetchall(), [(1,
-            None)])
+        testing.db.execute(users_autoinc.insert().
+                    values(user_name=bindparam('name', None)), [])
+        eq_(testing.db.execute(users_autoinc.select()).fetchall(), [(1, None)])
 
     @testing.requires.ad_hoc_engines
     def test_engine_level_options(self):
-        eng = engines.testing_engine(options={'execution_options'
-                : {'foo': 'bar'}})
+        eng = engines.testing_engine(options={'execution_options':
+                                            {'foo': 'bar'}})
         conn = eng.contextual_connect()
         eq_(conn._execution_options['foo'], 'bar')
         eq_(conn.execution_options(bat='hoho')._execution_options['foo'
@@ -301,6 +375,66 @@ class ExecuteTest(fixtures.TestBase):
         conn = eng.contextual_connect()
         eq_(conn._execution_options['foo'], 'hoho')
 
+    @testing.requires.ad_hoc_engines
+    def test_generative_engine_execution_options(self):
+        eng = engines.testing_engine(options={'execution_options':
+                                            {'base': 'x1'}})
+
+        eng1 = eng.execution_options(foo="b1")
+        eng2 = eng.execution_options(foo="b2")
+        eng1a = eng1.execution_options(bar="a1")
+        eng2a = eng2.execution_options(foo="b3", bar="a2")
+
+        eq_(eng._execution_options,
+                {'base': 'x1'})
+        eq_(eng1._execution_options,
+                {'base': 'x1', 'foo': 'b1'})
+        eq_(eng2._execution_options,
+                {'base': 'x1', 'foo': 'b2'})
+        eq_(eng1a._execution_options,
+                {'base': 'x1', 'foo': 'b1', 'bar': 'a1'})
+        eq_(eng2a._execution_options,
+                {'base': 'x1', 'foo': 'b3', 'bar': 'a2'})
+        is_(eng1a.pool, eng.pool)
+
+        # test pool is shared
+        eng2.dispose()
+        is_(eng1a.pool, eng2.pool)
+        is_(eng.pool, eng2.pool)
+
+    @testing.requires.ad_hoc_engines
+    def test_generative_engine_event_dispatch(self):
+        canary = []
+        def l1(*arg, **kw):
+            canary.append("l1")
+        def l2(*arg, **kw):
+            canary.append("l2")
+        def l3(*arg, **kw):
+            canary.append("l3")
+
+        eng = engines.testing_engine(options={'execution_options':
+                                            {'base': 'x1'}})
+        event.listen(eng, "before_execute", l1)
+
+        eng1 = eng.execution_options(foo="b1")
+        event.listen(eng, "before_execute", l2)
+        event.listen(eng1, "before_execute", l3)
+
+        eng.execute(select([1]))
+        eng1.execute(select([1]))
+
+        eq_(canary, ["l1", "l2", "l3", "l1", "l2"])
+
+    @testing.requires.ad_hoc_engines
+    def test_generative_engine_event_dispatch_hasevents(self):
+        def l1(*arg, **kw):
+            pass
+        eng = create_engine(testing.db.url)
+        assert not eng._has_events
+        event.listen(eng, "before_execute", l1)
+        eng2 = eng.execution_options(foo='bar')
+        assert eng2._has_events
+
     def test_unicode_test_fails_warning(self):
         class MockCursor(engines.DBAPIProxyCursor):
             def execute(self, stmt, params=None, **kw):
@@ -360,26 +494,28 @@ class ConvenienceExecuteTest(fixtures.TablesTest):
 
     def test_transaction_engine_ctx_begin_fails(self):
         engine = engines.testing_engine()
-        class MockConnection(Connection):
-            closed = False
-            def begin(self):
-                raise Exception("boom")
 
-            def close(self):
-                MockConnection.closed = True
-        engine._connection_cls = MockConnection
-        fn = self._trans_fn()
+        mock_connection = Mock(
+            return_value=Mock(
+                        begin=Mock(side_effect=Exception("boom"))
+                    )
+        )
+        engine._connection_cls = mock_connection
         assert_raises(
-            Exception, 
+            Exception,
             engine.begin
         )
-        assert MockConnection.closed
+
+        eq_(
+            mock_connection.return_value.close.mock_calls,
+            [call()]
+        )
 
     def test_transaction_engine_ctx_rollback(self):
         fn = self._trans_rollback_fn()
         ctx = testing.db.begin()
         assert_raises_message(
-            Exception, 
+            Exception,
             "breakage",
             testing.run_as_contextmanager, ctx, fn, 5, value=8
         )
@@ -388,7 +524,7 @@ class ConvenienceExecuteTest(fixtures.TablesTest):
     def test_transaction_tlocal_engine_ctx_commit(self):
         fn = self._trans_fn()
         engine = engines.testing_engine(options=dict(
-                                strategy='threadlocal', 
+                                strategy='threadlocal',
                                 pool=testing.db.pool))
         ctx = engine.begin()
         testing.run_as_contextmanager(ctx, fn, 5, value=8)
@@ -397,11 +533,11 @@ class ConvenienceExecuteTest(fixtures.TablesTest):
     def test_transaction_tlocal_engine_ctx_rollback(self):
         fn = self._trans_rollback_fn()
         engine = engines.testing_engine(options=dict(
-                                strategy='threadlocal', 
+                                strategy='threadlocal',
                                 pool=testing.db.pool))
         ctx = engine.begin()
         assert_raises_message(
-            Exception, 
+            Exception,
             "breakage",
             testing.run_as_contextmanager, ctx, fn, 5, value=8
         )
@@ -419,7 +555,7 @@ class ConvenienceExecuteTest(fixtures.TablesTest):
         conn = testing.db.connect()
         ctx = conn.begin()
         assert_raises_message(
-            Exception, 
+            Exception,
             "breakage",
             testing.run_as_contextmanager, ctx, fn, 5, value=8
         )
@@ -449,7 +585,7 @@ class ConvenienceExecuteTest(fixtures.TablesTest):
     def test_transaction_engine_fn_rollback(self):
         fn = self._trans_rollback_fn()
         assert_raises_message(
-            Exception, 
+            Exception,
             "breakage",
             testing.db.transaction, fn, 5, value=8
         )
@@ -465,7 +601,7 @@ class ConvenienceExecuteTest(fixtures.TablesTest):
         fn = self._trans_rollback_fn()
         conn = testing.db.connect()
         assert_raises(
-            Exception, 
+            Exception,
             conn.transaction, fn, 5, value=8
         )
         self._assert_no_data()
@@ -526,7 +662,7 @@ class LogParamsTest(fixtures.TestBase):
 
     def test_log_large_dict(self):
         self.eng.execute(
-            "INSERT INTO foo (data) values (:data)", 
+            "INSERT INTO foo (data) values (:data)",
             [{"data":str(i)} for i in xrange(100)]
         )
         eq_(
@@ -539,7 +675,7 @@ class LogParamsTest(fixtures.TestBase):
 
     def test_log_large_list(self):
         self.eng.execute(
-            "INSERT INTO foo (data) values (?)", 
+            "INSERT INTO foo (data) values (?)",
             [(str(i), ) for i in xrange(100)]
         )
         eq_(
@@ -558,7 +694,7 @@ class LogParamsTest(fixtures.TestBase):
             "{'data': '6'}, {'data': '7'}  ... displaying 10 of "
             "100 total bound parameter sets ...  {'data': '98'}, {'data': '99'}\]",
             lambda: self.eng.execute(
-                "INSERT INTO nonexistent (data) values (:data)", 
+                "INSERT INTO nonexistent (data) values (:data)",
                 [{"data":str(i)} for i in xrange(100)]
             )
         )
@@ -572,7 +708,7 @@ class LogParamsTest(fixtures.TestBase):
             "10 of 100 total bound parameter sets ...  "
             "\('98',\), \('99',\)\]",
             lambda: self.eng.execute(
-                "INSERT INTO nonexistent (data) values (?)", 
+                "INSERT INTO nonexistent (data) values (?)",
                 [(str(i), ) for i in xrange(100)]
             )
         )
@@ -586,7 +722,7 @@ class LoggingNameTest(fixtures.TestBase):
         for name in [b.name for b in self.buf.buffer]:
             assert name in (
                 'sqlalchemy.engine.base.Engine.%s' % eng_name,
-                'sqlalchemy.pool.%s.%s' % 
+                'sqlalchemy.pool.%s.%s' %
                     (eng.pool.__class__.__name__, pool_name)
             )
 
@@ -768,7 +904,7 @@ class MockStrategyTest(fixtures.TestBase):
 class ResultProxyTest(fixtures.TestBase):
 
     def test_nontuple_row(self):
-        """ensure the C version of BaseRowProxy handles 
+        """ensure the C version of BaseRowProxy handles
         duck-type-dependent rows."""
 
         from sqlalchemy.engine import RowProxy
@@ -814,9 +950,9 @@ class ResultProxyTest(fixtures.TestBase):
                 assert False
 
         execution_ctx_cls = engine.dialect.execution_ctx_cls
-        engine.dialect.execution_ctx_cls = type("FakeCtx", 
-                                            (BreakRowcountMixin, 
-                                            execution_ctx_cls), 
+        engine.dialect.execution_ctx_cls = type("FakeCtx",
+                                            (BreakRowcountMixin,
+                                            execution_ctx_cls),
                                             {})
 
         try:
@@ -863,6 +999,60 @@ class ResultProxyTest(fixtures.TestBase):
         writer.writerow(row)
         assert s.getvalue().strip() == '1,Test'
 
+    @testing.requires.selectone
+    def test_empty_accessors(self):
+        statements = [
+            (
+                "select 1",
+                [
+                    lambda r: r.last_inserted_params(),
+                    lambda r: r.last_updated_params(),
+                    lambda r: r.prefetch_cols(),
+                    lambda r: r.postfetch_cols(),
+                    lambda r : r.inserted_primary_key
+                ],
+                "Statement is not a compiled expression construct."
+            ),
+            (
+                select([1]),
+                [
+                    lambda r: r.last_inserted_params(),
+                    lambda r : r.inserted_primary_key
+                ],
+                r"Statement is not an insert\(\) expression construct."
+            ),
+            (
+                select([1]),
+                [
+                    lambda r: r.last_updated_params(),
+                ],
+                r"Statement is not an update\(\) expression construct."
+            ),
+            (
+                select([1]),
+                [
+                    lambda r: r.prefetch_cols(),
+                    lambda r : r.postfetch_cols()
+                ],
+                r"Statement is not an insert\(\) "
+                r"or update\(\) expression construct."
+            ),
+        ]
+
+        for stmt, meths, msg in statements:
+            r = testing.db.execute(stmt)
+            try:
+                for meth in meths:
+                    assert_raises_message(
+                        tsa.exc.InvalidRequestError,
+                        msg,
+                        meth, r
+                    )
+
+            finally:
+                r.close()
+
+
 class AlternateResultProxyTest(fixtures.TestBase):
     __requires__ = ('sqlite', )
 
@@ -871,7 +1061,7 @@ class AlternateResultProxyTest(fixtures.TestBase):
         from sqlalchemy.engine import base, default
         cls.engine = engine = testing_engine('sqlite://')
         m = MetaData()
-        cls.table = t = Table('test', m, 
+        cls.table = t = Table('test', m,
             Column('x', Integer, primary_key=True),
             Column('y', String(50, convert_unicode='force'))
         )
@@ -913,27 +1103,29 @@ class AlternateResultProxyTest(fixtures.TestBase):
         eq_(rows, [(i, "t_%d" % i) for i in xrange(1, 6)])
 
     def test_plain(self):
-        self._test_proxy(base.ResultProxy)
+        self._test_proxy(_result.ResultProxy)
 
     def test_buffered_row_result_proxy(self):
-        self._test_proxy(base.BufferedRowResultProxy)
+        self._test_proxy(_result.BufferedRowResultProxy)
 
     def test_fully_buffered_result_proxy(self):
-        self._test_proxy(base.FullyBufferedResultProxy)
+        self._test_proxy(_result.FullyBufferedResultProxy)
 
     def test_buffered_column_result_proxy(self):
-        self._test_proxy(base.BufferedColumnResultProxy)
+        self._test_proxy(_result.BufferedColumnResultProxy)
 
 class EngineEventsTest(fixtures.TestBase):
-    __requires__ = 'ad_hoc_engines', 
+    __requires__ = 'ad_hoc_engines',
 
     def tearDown(self):
         Engine.dispatch._clear()
+        Engine._has_events = False
 
     def _assert_stmts(self, expected, received):
+        orig = list(received)
         for stmt, params, posn in expected:
             if not received:
-                assert False
+                assert False, "Nothing available for stmt: %s" % stmt
             while received:
                 teststmt, testparams, testmultiparams = \
                     received.pop(0)
@@ -985,6 +1177,26 @@ class EngineEventsTest(fixtures.TestBase):
 
         eq_(canary, ['be1', 'be3', 'be2', 'be1', 'be3'])
 
+    def test_per_connection_plus_engine(self):
+        canary = []
+        def be1(conn, stmt, *arg):
+            canary.append('be1')
+        def be2(conn, stmt, *arg):
+            canary.append('be2')
+        e1 = testing_engine(config.db_url)
+
+        event.listen(e1, "before_execute", be1)
+
+        conn = e1.connect()
+        event.listen(conn, "before_execute", be2)
+        canary[:] = []
+        conn.execute(select([1]))
+
+        eq_(canary, ['be2', 'be1'])
+
+        conn._branch().execute(select([1]))
+        eq_(canary, ['be2', 'be1', 'be2', 'be1'])
+
     def test_argument_format_execute(self):
         def before_execute(conn, clauseelement, multiparams, params):
             assert isinstance(multiparams, (list, tuple))
@@ -999,7 +1211,7 @@ class EngineEventsTest(fixtures.TestBase):
         e1.execute(select([1]))
         e1.execute(select([1]).compile(dialect=e1.dialect).statement)
         e1.execute(select([1]).compile(dialect=e1.dialect))
-        e1._execute_compiled(select([1]).compile(dialect=e1.dialect), [], {})
+        e1._execute_compiled(select([1]).compile(dialect=e1.dialect), (), {})
 
     def test_exception_event(self):
         engine = engines.testing_engine()
@@ -1028,22 +1240,23 @@ class EngineEventsTest(fixtures.TestBase):
                                                     params ):
             stmts.append((str(clauseelement), params, multiparams))
 
-        def cursor_execute(conn, cursor, statement, parameters, 
+        def cursor_execute(conn, cursor, statement, parameters,
                                 context, executemany):
             cursor_stmts.append((str(statement), parameters, None))
 
 
         for engine in [
-            engines.testing_engine(options=dict(implicit_returning=False)), 
+            engines.testing_engine(options=dict(implicit_returning=False)),
             engines.testing_engine(options=dict(implicit_returning=False,
-                                   strategy='threadlocal'))
+                                   strategy='threadlocal')),
+            engines.testing_engine(options=dict(implicit_returning=False)).\
+                connect()
             ]:
             event.listen(engine, 'before_execute', execute)
             event.listen(engine, 'before_cursor_execute', cursor_execute)
-
             m = MetaData(engine)
-            t1 = Table('t1', m, 
-                Column('c1', Integer, primary_key=True), 
+            t1 = Table('t1', m,
+                Column('c1', Integer, primary_key=True),
                 Column('c2', String(50), default=func.lower('Foo'),
                                             primary_key=True)
             )
@@ -1055,21 +1268,26 @@ class EngineEventsTest(fixtures.TestBase):
                     'some data'), (6, 'foo')])
             finally:
                 m.drop_all()
-            engine.dispose()
+
             compiled = [('CREATE TABLE t1', {}, None),
-                        ('INSERT INTO t1 (c1, c2)', {'c2': 'some data',
-                        'c1': 5}, None), ('INSERT INTO t1 (c1, c2)',
-                        {'c1': 6}, None), ('select * from t1', {},
-                        None), ('DROP TABLE t1', {}, None)]
-            if not testing.against('oracle+zxjdbc'):  # or engine.dialect.pr
-                                                      # eexecute_pk_sequence
-                                                      # s:
+                        ('INSERT INTO t1 (c1, c2)',
+                                {'c2': 'some data', 'c1': 5}, None),
+                        ('INSERT INTO t1 (c1, c2)',
+                        {'c1': 6}, None),
+                        ('select * from t1', {}, None),
+                        ('DROP TABLE t1', {}, None)]
+
+            # or engine.dialect.preexecute_pk_sequences:
+            if not testing.against('oracle+zxjdbc'):
                 cursor = [
                     ('CREATE TABLE t1', {}, ()),
-                    ('INSERT INTO t1 (c1, c2)', {'c2': 'some data', 'c1'
-                     : 5}, (5, 'some data')),
-                    ('SELECT lower', {'lower_2': 'Foo'}, ('Foo', )),
-                    ('INSERT INTO t1 (c1, c2)', {'c2': 'foo', 'c1': 6},
+                    ('INSERT INTO t1 (c1, c2)', {
+                        'c2': 'some data', 'c1': 5},
+                        (5, 'some data')),
+                    ('SELECT lower', {'lower_2': 'Foo'},
+                        ('Foo', )),
+                    ('INSERT INTO t1 (c1, c2)',
+                     {'c2': 'foo', 'c1': 6},
                      (6, 'foo')),
                     ('select * from t1', {}, ()),
                     ('DROP TABLE t1', {}, ()),
@@ -1079,18 +1297,20 @@ class EngineEventsTest(fixtures.TestBase):
                 if testing.against('oracle+zxjdbc'):
                     insert2_params += (ReturningParam(12), )
                 cursor = [('CREATE TABLE t1', {}, ()),
-                          ('INSERT INTO t1 (c1, c2)', {'c2': 'some data'
-                          , 'c1': 5}, (5, 'some data')),
+                          ('INSERT INTO t1 (c1, c2)',
+                            {'c2': 'some data', 'c1': 5}, (5, 'some data')),
                           ('INSERT INTO t1 (c1, c2)', {'c1': 6,
                           'lower_2': 'Foo'}, insert2_params),
-                          ('select * from t1', {}, ()), ('DROP TABLE t1'
-                          , {}, ())]  # bind param name 'lower_2' might
-                                      # be incorrect
+                          ('select * from t1', {}, ()),
+                          ('DROP TABLE t1', {}, ())]
+                                # bind param name 'lower_2' might
+                                # be incorrect
             self._assert_stmts(compiled, stmts)
             self._assert_stmts(cursor, cursor_stmts)
 
     def test_options(self):
         canary = []
+
         def execute(conn, *args, **kw):
             canary.append('execute')
 
@@ -1119,7 +1339,7 @@ class EngineEventsTest(fixtures.TestBase):
             canary.append('execute')
             return clauseelement, multiparams, params
 
-        def cursor_execute(conn, cursor, statement, 
+        def cursor_execute(conn, cursor, statement,
                         parameters, context, executemany):
             canary.append('cursor_execute')
             return statement, parameters
@@ -1138,6 +1358,33 @@ class EngineEventsTest(fixtures.TestBase):
             canary, ['execute', 'cursor_execute']
         )
 
+    @testing.requires.sequences
+    @testing.provide_metadata
+    def test_cursor_execute(self):
+        canary = []
+        def tracker(name):
+            def go(conn, cursor, statement, parameters, context, executemany):
+                canary.append((statement, context))
+            return go
+        engine = engines.testing_engine()
+
+
+        t = Table('t', self.metadata,
+                    Column('x', Integer, Sequence('t_id_seq'), primary_key=True),
+                    implicit_returning=False
+                    )
+        self.metadata.create_all(engine)
+        with engine.begin() as conn:
+            event.listen(conn, 'before_cursor_execute', tracker('cursor_execute'))
+            conn.execute(t.insert())
+        # we see the sequence pre-executed in the first call
+        assert "t_id_seq" in canary[0][0]
+        assert "INSERT" in canary[1][0]
+        # same context
+        is_(
+            canary[0][1], canary[1][1]
+        )
+
     def test_transactional(self):
         canary = []
         def tracker(name):
@@ -1168,20 +1415,30 @@ class EngineEventsTest(fixtures.TestBase):
     @testing.requires.savepoints
     @testing.requires.two_phase_transactions
     def test_transactional_advanced(self):
-        canary = []
-        def tracker(name):
+        canary1 = []
+        def tracker1(name):
             def go(*args, **kw):
-                canary.append(name)
+                canary1.append(name)
+            return go
+        canary2 = []
+        def tracker2(name):
+            def go(*args, **kw):
+                canary2.append(name)
             return go
 
         engine = engines.testing_engine()
-        for name in ['begin', 'savepoint', 
+        for name in ['begin', 'savepoint',
                     'rollback_savepoint', 'release_savepoint',
-                    'rollback', 'begin_twophase', 
+                    'rollback', 'begin_twophase',
                        'prepare_twophase', 'commit_twophase']:
-            event.listen(engine, '%s' % name, tracker(name))
+            event.listen(engine, '%s' % name, tracker1(name))
 
         conn = engine.connect()
+        for name in ['begin', 'savepoint',
+                    'rollback_savepoint', 'release_savepoint',
+                    'rollback', 'begin_twophase',
+                       'prepare_twophase', 'commit_twophase']:
+            event.listen(conn, '%s' % name, tracker2(name))
 
         trans = conn.begin()
         trans2 = conn.begin_nested()
@@ -1197,9 +1454,14 @@ class EngineEventsTest(fixtures.TestBase):
         trans.prepare()
         trans.commit()
 
-        eq_(canary, ['begin', 'savepoint', 
+        eq_(canary1, ['begin', 'savepoint',
                     'rollback_savepoint', 'savepoint', 'release_savepoint',
-                    'rollback', 'begin_twophase', 
+                    'rollback', 'begin_twophase',
+                       'prepare_twophase', 'commit_twophase']
+        )
+        eq_(canary2, ['begin', 'savepoint',
+                    'rollback_savepoint', 'savepoint', 'release_savepoint',
+                    'rollback', 'begin_twophase',
                        'prepare_twophase', 'commit_twophase']
         )
 
@@ -1209,7 +1471,7 @@ class ProxyConnectionTest(fixtures.TestBase):
     the deprecated ConnectionProxy interface.
 
     """
-    __requires__ = 'ad_hoc_engines', 
+    __requires__ = 'ad_hoc_engines',
 
     @testing.uses_deprecated(r'.*Use event.listen')
     @testing.fails_on('firebird', 'Data type unknown')
@@ -1245,7 +1507,7 @@ class ProxyConnectionTest(fixtures.TestBase):
         def assert_stmts(expected, received):
             for stmt, params, posn in expected:
                 if not received:
-                    assert False
+                    assert False, "Nothing available for stmt: %s" % stmt
                 while received:
                     teststmt, testparams, testmultiparams = \
                         received.pop(0)
@@ -1262,8 +1524,8 @@ class ProxyConnectionTest(fixtures.TestBase):
                                    proxy=MyProxy(),
                                    strategy='threadlocal')):
             m = MetaData(engine)
-            t1 = Table('t1', m, 
-                Column('c1', Integer, primary_key=True), 
+            t1 = Table('t1', m,
+                Column('c1', Integer, primary_key=True),
                 Column('c2', String(50), default=func.lower('Foo'),
                                             primary_key=True)
             )
@@ -1288,7 +1550,8 @@ class ProxyConnectionTest(fixtures.TestBase):
                     ('CREATE TABLE t1', {}, ()),
                     ('INSERT INTO t1 (c1, c2)', {'c2': 'some data', 'c1'
                      : 5}, (5, 'some data')),
-                    ('SELECT lower', {'lower_2': 'Foo'}, ('Foo', )),
+                    ('SELECT lower', {'lower_2': 'Foo'},
+                        ('Foo', )),
                     ('INSERT INTO t1 (c1, c2)', {'c2': 'foo', 'c1': 6},
                      (6, 'foo')),
                     ('select * from t1', {}, ()),
@@ -1385,9 +1648,9 @@ class ProxyConnectionTest(fixtures.TestBase):
         trans.commit()
 
         canary = [t for t in canary if t not in ('cursor_execute', 'execute')]
-        eq_(canary, ['begin', 'savepoint', 
+        eq_(canary, ['begin', 'savepoint',
                     'rollback_savepoint', 'savepoint', 'release_savepoint',
-                    'rollback', 'begin_twophase', 
+                    'rollback', 'begin_twophase',
                        'prepare_twophase', 'commit_twophase']
         )
 
