@@ -1,82 +1,123 @@
-from test.lib.testing import eq_, assert_raises, assert_raises_message
-import time
-import weakref
-from sqlalchemy import select, MetaData, Integer, String, pool, create_engine
-from test.lib.schema import Table, Column
-import sqlalchemy as tsa
-from test.lib import testing, engines
-from test.lib.util import gc_collect
-from sqlalchemy import exc
-from test.lib import fixtures
-from test.lib.engines import testing_engine
+from __future__ import with_statement
 
-class MockDisconnect(Exception):
+from sqlalchemy.testing import eq_, assert_raises, assert_raises_message
+import time
+from sqlalchemy import select, MetaData, Integer, String, create_engine, pool
+from sqlalchemy.testing.schema import Table, Column
+import sqlalchemy as tsa
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
+from sqlalchemy.testing.util import gc_collect
+from sqlalchemy import exc
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.engines import testing_engine
+from sqlalchemy.testing import is_not_
+from sqlalchemy.testing.mock import Mock, call
+from sqlalchemy import util
+
+class MockError(Exception):
     pass
 
-class MockDBAPI(object):
-    def __init__(self):
-        self.paramstyle = 'named'
-        self.connections = weakref.WeakKeyDictionary()
-    def connect(self, *args, **kwargs):
-        return MockConnection(self)
-    def shutdown(self):
-        for c in self.connections:
-            c.explode[0] = True
-    Error = MockDisconnect
+class MockDisconnect(MockError):
+    pass
 
-class MockConnection(object):
-    def __init__(self, dbapi):
-        dbapi.connections[self] = True
-        self.explode = [False]
-    def rollback(self):
-        pass
-    def commit(self):
-        pass
-    def cursor(self):
-        return MockCursor(self)
-    def close(self):
-        pass
+def mock_connection():
+    def mock_cursor():
+        def execute(*args, **kwargs):
+            if conn.explode == 'execute':
+                raise MockDisconnect("Lost the DB connection on execute")
+            elif conn.explode in ('execute_no_disconnect', ):
+                raise MockError(
+                    "something broke on execute but we didn't lose the connection")
+            elif conn.explode in ('rollback', 'rollback_no_disconnect'):
+                raise MockError(
+                    "something broke on execute but we didn't lose the connection")
+            elif args and "SELECT" in args[0]:
+                cursor.description = [('foo', None, None, None, None, None)]
+            else:
+                return
 
-class MockCursor(object):
-    def __init__(self, parent):
-        self.explode = parent.explode
-        self.description = ()
-    def execute(self, *args, **kwargs):
-        if self.explode[0]:
-            raise MockDisconnect("Lost the DB connection")
+        def close():
+            cursor.fetchall = cursor.fetchone = \
+                Mock(side_effect=MockError("cursor closed"))
+        cursor = Mock(
+                    execute=Mock(side_effect=execute),
+                    close=Mock(side_effect=close)
+                )
+        return cursor
+
+    def cursor():
+        while True:
+            yield mock_cursor()
+
+    def rollback():
+        if conn.explode == 'rollback':
+            raise MockDisconnect("Lost the DB connection on rollback")
+        if conn.explode == 'rollback_no_disconnect':
+            raise MockError(
+                "something broke on rollback but we didn't lose the connection")
         else:
             return
-    def close(self):
-        pass
 
-db, dbapi = None, None
+    conn = Mock(
+                rollback=Mock(side_effect=rollback),
+                cursor=Mock(side_effect=cursor())
+            )
+    return conn
+
+def MockDBAPI():
+    connections = []
+    def connect():
+        while True:
+            conn = mock_connection()
+            connections.append(conn)
+            yield conn
+
+    def shutdown(explode='execute'):
+        for c in connections:
+            c.explode = explode
+
+    def dispose():
+        for c in connections:
+            c.explode = None
+        connections[:] = []
+
+    return Mock(
+                connect=Mock(side_effect=connect()),
+                shutdown=Mock(side_effect=shutdown),
+                dispose=Mock(side_effect=dispose),
+                paramstyle='named',
+                connections=connections,
+                Error=MockError
+            )
+
+
 class MockReconnectTest(fixtures.TestBase):
     def setup(self):
-        global db, dbapi
-        dbapi = MockDBAPI()
+        self.dbapi = MockDBAPI()
 
-        # note - using straight create_engine here
-        # since we are testing gc
-        db = create_engine(
-                    'postgresql://foo:bar@localhost/test', 
-                    module=dbapi, _initialize=False)
+        self.db = testing_engine(
+                    'postgresql://foo:bar@localhost/test',
+                    options=dict(module=self.dbapi, _initialize=False))
 
+        self.mock_connect = call(host='localhost', password='bar',
+                                user='foo', database='test')
         # monkeypatch disconnect checker
-        db.dialect.is_disconnect = lambda e, conn, cursor: isinstance(e, MockDisconnect)
+        self.db.dialect.is_disconnect = lambda e, conn, cursor: isinstance(e, MockDisconnect)
 
     def teardown(self):
-        db.dispose()
+        self.dbapi.dispose()
 
     def test_reconnect(self):
         """test that an 'is_disconnect' condition will invalidate the
         connection, and additionally dispose the previous connection
         pool and recreate."""
 
-        pid = id(db.pool)
+        db_pool = self.db.pool
 
         # make a connection
 
-        conn = db.connect()
+        conn = self.db.connect()
 
         # connection works
 
@@ -85,55 +126,61 @@ class MockReconnectTest(fixtures.TestBase):
         # create a second connection within the pool, which we'll ensure
         # also goes away
 
-        conn2 = db.connect()
+        conn2 = self.db.connect()
         conn2.close()
 
         # two connections opened total now
 
-        assert len(dbapi.connections) == 2
+        assert len(self.dbapi.connections) == 2
 
         # set it to fail
 
-        dbapi.shutdown()
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError:
-            pass
+        self.dbapi.shutdown()
+        assert_raises(
+            tsa.exc.DBAPIError,
+            conn.execute, select([1])
+        )
 
         # assert was invalidated
 
         assert not conn.closed
         assert conn.invalidated
 
-        # close shouldnt break
+        # close shouldn't break
 
         conn.close()
-        assert id(db.pool) != pid
+        is_not_(self.db.pool, db_pool)
 
         # ensure all connections closed (pool was recycled)
 
-        gc_collect()
-        assert len(dbapi.connections) == 0
-        conn = db.connect()
+        eq_(
+            [c.close.mock_calls for c in self.dbapi.connections],
+            [[call()], [call()]]
+        )
+
+        conn = self.db.connect()
         conn.execute(select([1]))
         conn.close()
-        assert len(dbapi.connections) == 1
+
+        eq_(
+            [c.close.mock_calls for c in self.dbapi.connections],
+            [[call()], [call()], []]
+        )
 
     def test_invalidate_trans(self):
-        conn = db.connect()
+        conn = self.db.connect()
         trans = conn.begin()
-        dbapi.shutdown()
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError:
-            pass
+        self.dbapi.shutdown()
 
-        # assert was invalidated
+        assert_raises(
+            tsa.exc.DBAPIError,
+            conn.execute, select([1])
+        )
 
-        gc_collect()
-        assert len(dbapi.connections) == 0
+        eq_(
+            [c.close.mock_calls for c in self.dbapi.connections],
+            [[call()]]
+        )
         assert not conn.closed
         assert conn.invalidated
         assert trans.is_active
@@ -143,105 +190,243 @@ class MockReconnectTest(fixtures.TestBase):
             conn.execute, select([1])
         )
         assert trans.is_active
-        try:
-            trans.commit()
-            assert False
-        except tsa.exc.InvalidRequestError, e:
-            assert str(e) \
-                == "Can't reconnect until invalid transaction is "\
-                "rolled back"
+
+        assert_raises_message(
+            tsa.exc.InvalidRequestError,
+            "Can't reconnect until invalid transaction is "
+                "rolled back",
+            trans.commit
+        )
+
         assert trans.is_active
         trans.rollback()
         assert not trans.is_active
         conn.execute(select([1]))
         assert not conn.invalidated
-        assert len(dbapi.connections) == 1
+        eq_(
+            [c.close.mock_calls for c in self.dbapi.connections],
+            [[call()], []]
+        )
 
     def test_conn_reusable(self):
-        conn = db.connect()
+        conn = self.db.connect()
 
         conn.execute(select([1]))
 
-        assert len(dbapi.connections) == 1
+        eq_(
+            self.dbapi.connect.mock_calls,
+            [self.mock_connect]
+        )
 
-        dbapi.shutdown()
+        self.dbapi.shutdown()
 
-        # raises error
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError:
-            pass
+        assert_raises(
+            tsa.exc.DBAPIError,
+            conn.execute, select([1])
+        )
 
         assert not conn.closed
         assert conn.invalidated
 
-        # ensure all connections closed (pool was recycled)
-        gc_collect()
-        assert len(dbapi.connections) == 0
+        eq_(
+            [c.close.mock_calls for c in self.dbapi.connections],
+            [[call()]]
+        )
 
         # test reconnects
         conn.execute(select([1]))
         assert not conn.invalidated
-        assert len(dbapi.connections) == 1
+
+        eq_(
+            [c.close.mock_calls for c in self.dbapi.connections],
+            [[call()], []]
+        )
+
+    def test_invalidated_close(self):
+        conn = self.db.connect()
+
+        self.dbapi.shutdown()
+
+        assert_raises(
+            tsa.exc.DBAPIError,
+            conn.execute, select([1])
+        )
+
+        conn.close()
+        assert conn.closed
+        assert conn.invalidated
+        assert_raises_message(
+            tsa.exc.StatementError,
+            "This Connection is closed",
+            conn.execute, select([1])
+        )
+
+    def test_noreconnect_execute_plus_closewresult(self):
+        conn = self.db.connect(close_with_result=True)
+
+        self.dbapi.shutdown("execute_no_disconnect")
+
+        # raises error
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            "something broke on execute but we didn't lose the connection",
+            conn.execute, select([1])
+        )
+
+        assert conn.closed
+        assert not conn.invalidated
+
+    def test_noreconnect_rollback_plus_closewresult(self):
+        conn = self.db.connect(close_with_result=True)
+
+        self.dbapi.shutdown("rollback_no_disconnect")
+
+        # raises error
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            "something broke on rollback but we didn't lose the connection",
+            conn.execute, select([1])
+        )
+
+        assert conn.closed
+        assert not conn.invalidated
+
+        assert_raises_message(
+            tsa.exc.StatementError,
+            "This Connection is closed",
+            conn.execute, select([1])
+        )
+
+    def test_reconnect_on_reentrant(self):
+        conn = self.db.connect()
+
+        conn.execute(select([1]))
+
+        assert len(self.dbapi.connections) == 1
+
+        self.dbapi.shutdown("rollback")
+
+        # raises error
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            "Lost the DB connection on rollback",
+            conn.execute, select([1])
+        )
+
+        assert not conn.closed
+        assert conn.invalidated
+
+    def test_reconnect_on_reentrant_plus_closewresult(self):
+        conn = self.db.connect(close_with_result=True)
+
+        self.dbapi.shutdown("rollback")
+
+        # raises error
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            "Lost the DB connection on rollback",
+            conn.execute, select([1])
+        )
+
+        assert conn.closed
+        assert conn.invalidated
+
+        assert_raises_message(
+            tsa.exc.StatementError,
+            "This Connection is closed",
+            conn.execute, select([1])
+        )
+
+    def test_check_disconnect_no_cursor(self):
+        conn = self.db.connect()
+        result = conn.execute(select([1]))
+        result.cursor.close()
+        conn.close()
+
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            "cursor closed",
+            list, result
+        )
+
+    def test_dialect_initialize_once(self):
+        from sqlalchemy.engine.base import Engine
+        from sqlalchemy.engine.url import URL
+        from sqlalchemy.engine.default import DefaultDialect
+        from sqlalchemy.pool import QueuePool
+        dbapi = self.dbapi
+
+        mock_dialect = Mock()
+        class MyURL(URL):
+            def get_dialect(self):
+                return Dialect
+        class Dialect(DefaultDialect):
+            initialize = Mock()
+
+        engine = create_engine(MyURL("foo://"), module=dbapi)
+        c1 = engine.connect()
+        engine.dispose()
+        c2 = engine.connect()
+        eq_(Dialect.initialize.call_count, 1)
+
+
 
 class CursorErrTest(fixtures.TestBase):
 
     def setup(self):
-        global db, dbapi
+        def MockDBAPI():
+            def cursor():
+                while True:
+                    yield Mock(
+                        description=[],
+                        close=Mock(side_effect=Exception("explode")))
+            def connect():
+                while True:
+                    yield Mock(cursor=Mock(side_effect=cursor()))
 
-        class MDBAPI(MockDBAPI):
-            def connect(self, *args, **kwargs):
-                return MConn(self)
+            return Mock(connect=Mock(side_effect=connect()))
 
-        class MConn(MockConnection):
-            def cursor(self):
-                return MCursor(self)
-
-        class MCursor(MockCursor):
-            def close(self):
-                raise Exception("explode")
-
-        dbapi = MDBAPI()
-
-        db = testing_engine(
-                    'postgresql://foo:bar@localhost/test', 
+        dbapi = MockDBAPI()
+        self.db = testing_engine(
+                    'postgresql://foo:bar@localhost/test',
                     options=dict(module=dbapi, _initialize=False))
 
     def test_cursor_explode(self):
-        conn = db.connect()
+        conn = self.db.connect()
         result = conn.execute("select foo")
         result.close()
         conn.close()
 
     def teardown(self):
-        db.dispose()
+        self.db.dispose()
+
+
+def _assert_invalidated(fn, *args):
+    try:
+        fn(*args)
+        assert False
+    except tsa.exc.DBAPIError, e:
+        if not e.connection_invalidated:
+            raise
 
-engine = None
 class RealReconnectTest(fixtures.TestBase):
     def setup(self):
-        global engine
-        engine = engines.reconnecting_engine()
+        self.engine = engines.reconnecting_engine()
 
     def teardown(self):
-        engine.dispose()
+        self.engine.dispose()
 
     @testing.fails_on('+informixdb',
                       "Wrong error thrown, fix in informixdb?")
     def test_reconnect(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
 
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.closed
 
-        engine.test_shutdown()
+        self.engine.test_shutdown()
 
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError, e:
-            if not e.connection_invalidated:
-                raise
+        _assert_invalidated(conn.execute, select([1]))
 
         assert not conn.closed
         assert conn.invalidated
@@ -251,19 +436,34 @@ class RealReconnectTest(fixtures.TestBase):
         assert not conn.invalidated
 
         # one more time
-        engine.test_shutdown()
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError, e:
-            if not e.connection_invalidated:
-                raise
+        self.engine.test_shutdown()
+        _assert_invalidated(conn.execute, select([1]))
+
         assert conn.invalidated
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.invalidated
 
         conn.close()
 
+    def test_multiple_invalidate(self):
+        c1 = self.engine.connect()
+        c2 = self.engine.connect()
+
+        eq_(c1.execute(select([1])).scalar(), 1)
+
+        p1 = self.engine.pool
+        self.engine.test_shutdown()
+
+        _assert_invalidated(c1.execute, select([1]))
+
+        p2 = self.engine.pool
+
+        _assert_invalidated(c2.execute, select([1]))
+
+        # pool isn't replaced
+        assert self.engine.pool is p2
+
+
     def test_ensure_is_disconnect_gets_connection(self):
         def is_disconnect(e, conn, cursor):
             # connection is still present
@@ -272,37 +472,37 @@ class RealReconnectTest(fixtures.TestBase):
             # though MySQLdb we get a non-working cursor.
             # assert cursor is None
 
-        engine.dialect.is_disconnect = is_disconnect
-        conn = engine.connect()
-        engine.test_shutdown()
+        self.engine.dialect.is_disconnect = is_disconnect
+        conn = self.engine.connect()
+        self.engine.test_shutdown()
         assert_raises(
             tsa.exc.DBAPIError,
             conn.execute, select([1])
         )
 
     def test_rollback_on_invalid_plain(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         trans = conn.begin()
         conn.invalidate()
         trans.rollback()
 
     @testing.requires.two_phase_transactions
     def test_rollback_on_invalid_twophase(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         trans = conn.begin_twophase()
         conn.invalidate()
         trans.rollback()
 
     @testing.requires.savepoints
     def test_rollback_on_invalid_savepoint(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         trans = conn.begin()
         trans2 = conn.begin_nested()
         conn.invalidate()
         trans2.rollback()
 
     def test_invalidate_twice(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         conn.invalidate()
         conn.invalidate()
 
@@ -316,9 +516,15 @@ class RealReconnectTest(fixtures.TestBase):
         # raises a DBAPIError, not an AttributeError
         assert_raises(exc.DBAPIError, engine.connect)
 
-        # dispose connections so we get a new one on
-        # next go
-        engine.dispose()
+    @testing.skip_if(
+        [lambda: util.py3k, "oracle+cx_oracle"],
+        "Crashes on py3k+cx_oracle")
+    def test_explode_in_initializer_disconnect(self):
+        engine = engines.testing_engine()
+        def broken_initialize(connection):
+            connection.execute("select fake_stuff from _fake_table")
+
+        engine.dialect.initialize = broken_initialize
 
         p1 = engine.pool
 
@@ -342,12 +548,7 @@ class RealReconnectTest(fixtures.TestBase):
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.closed
         engine.test_shutdown()
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError, e:
-            if not e.connection_invalidated:
-                raise
+        _assert_invalidated(conn.execute, select([1]))
         assert not conn.closed
         assert conn.invalidated
         eq_(conn.execute(select([1])).scalar(), 1)
@@ -356,37 +557,27 @@ class RealReconnectTest(fixtures.TestBase):
     @testing.fails_on('+informixdb',
                       "Wrong error thrown, fix in informixdb?")
     def test_close(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.closed
 
-        engine.test_shutdown()
+        self.engine.test_shutdown()
 
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError, e:
-            if not e.connection_invalidated:
-                raise
+        _assert_invalidated(conn.execute, select([1]))
 
         conn.close()
-        conn = engine.connect()
+        conn = self.engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
 
     @testing.fails_on('+informixdb',
                       "Wrong error thrown, fix in informixdb?")
     def test_with_transaction(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         trans = conn.begin()
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.closed
-        engine.test_shutdown()
-        try:
-            conn.execute(select([1]))
-            assert False
-        except tsa.exc.DBAPIError, e:
-            if not e.connection_invalidated:
-                raise
+        self.engine.test_shutdown()
+        _assert_invalidated(conn.execute, select([1]))
         assert not conn.closed
         assert conn.invalidated
         assert trans.is_active
@@ -397,13 +588,11 @@ class RealReconnectTest(fixtures.TestBase):
             conn.execute, select([1])
         )
         assert trans.is_active
-        try:
-            trans.commit()
-            assert False
-        except tsa.exc.InvalidRequestError, e:
-            assert str(e) \
-                == "Can't reconnect until invalid transaction is "\
-                "rolled back"
+        assert_raises_message(
+            tsa.exc.InvalidRequestError,
+            "Can't reconnect until invalid transaction is rolled back",
+            trans.commit
+        )
         assert trans.is_active
         trans.rollback()
         assert not trans.is_active
@@ -424,7 +613,7 @@ class RecycleTest(fixtures.TestBase):
 
             # set the pool recycle down to 1.
             # we aren't doing this inline with the
-            # engine create since cx_oracle takes way 
+            # engine create since cx_oracle takes way
             # too long to create the 1st connection and don't
             # want to build a huge delay into this test.
 
@@ -441,46 +630,35 @@ class RecycleTest(fixtures.TestBase):
             eq_(conn.execute(select([1])).scalar(), 1)
             conn.close()
 
-meta, table, engine = None, None, None
 class InvalidateDuringResultTest(fixtures.TestBase):
     def setup(self):
-        global meta, table, engine
-        engine = engines.reconnecting_engine()
-        meta = MetaData(engine)
-        table = Table('sometable', meta,
+        self.engine = engines.reconnecting_engine()
+        self.meta = MetaData(self.engine)
+        table = Table('sometable', self.meta,
             Column('id', Integer, primary_key=True),
             Column('name', String(50)))
-        meta.create_all()
+        self.meta.create_all()
         table.insert().execute(
-            [{'id':i, 'name':'row %d' % i} for i in range(1, 100)]
+            [{'id': i, 'name': 'row %d' % i} for i in range(1, 100)]
         )
 
     def teardown(self):
-        meta.drop_all()
-        engine.dispose()
+        self.meta.drop_all()
+        self.engine.dispose()
 
-    @testing.fails_on('+pymysql',
-                      "Buffers the result set and doesn't check for "
-                      "connection close")
-    @testing.fails_on('+mysqldb',
-                      "Buffers the result set and doesn't check for "
-                      "connection close")
-    @testing.fails_on('+pg8000',
-                      "Buffers the result set and doesn't check for "
-                      "connection close")
+    @testing.fails_if([
+                    '+mysqlconnector', '+mysqldb',
+                    '+cymysql', '+pymysql', '+pg8000'
+                    ], "Buffers the result set and doesn't check for "
+                        "connection close")
     @testing.fails_on('+informixdb',
                       "Wrong error thrown, fix in informixdb?")
     def test_invalidate_on_results(self):
-        conn = engine.connect()
+        conn = self.engine.connect()
         result = conn.execute('select * from sometable')
         for x in xrange(20):
             result.fetchone()
-        engine.test_shutdown()
-        try:
-            print 'ghost result: %r' % result.fetchone()
-            assert False
-        except tsa.exc.DBAPIError, e:
-            if not e.connection_invalidated:
-                raise
+        self.engine.test_shutdown()
+        _assert_invalidated(result.fetchone)
         assert conn.invalidated
 
