@@ -1,15 +1,20 @@
 import warnings
-from test.lib.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing import eq_, assert_raises, assert_raises_message
 from sqlalchemy import *
 from sqlalchemy import exc as sa_exc, util, event
 from sqlalchemy.orm import *
+from sqlalchemy.orm.util import instance_str
 from sqlalchemy.orm import exc as orm_exc, attributes
-from test.lib.assertsql import AllOf, CompiledSQL
+from sqlalchemy.testing.assertsql import AllOf, CompiledSQL
 from sqlalchemy.sql import table, column
-from test.lib import testing, engines
-from test.lib import fixtures
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
+from sqlalchemy import inspect
+from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.testing.util import gc_collect
 
 class O2MTest(fixtures.MappedTest):
     """deals with inheritance and one-to-many relationships"""
@@ -68,11 +73,107 @@ class O2MTest(fixtures.MappedTest):
         l = sess.query(Blub).all()
         result = ','.join([repr(l[0]), repr(l[1]),
                           repr(l[0].parent_foo), repr(l[1].parent_foo)])
-        print compare
-        print result
-        self.assert_(compare == result)
-        self.assert_(l[0].parent_foo.data == 'foo #1'
-                     and l[1].parent_foo.data == 'foo #1')
+        eq_(compare, result)
+        eq_(l[0].parent_foo.data, 'foo #1')
+        eq_(l[1].parent_foo.data, 'foo #1')
+
+class PolymorphicResolutionMultiLevel(fixtures.DeclarativeMappedTest,
+                                        testing.AssertsCompiledSQL):
+    run_setup_mappers = 'once'
+    __dialect__ = 'default'
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+        class A(Base):
+            __tablename__ = 'a'
+            id = Column(Integer, primary_key=True)
+        class B(A):
+            __tablename__ = 'b'
+            id = Column(Integer, ForeignKey('a.id'), primary_key=True)
+        class C(A):
+            __tablename__ = 'c'
+            id = Column(Integer, ForeignKey('a.id'), primary_key=True)
+        class D(B):
+            __tablename__ = 'd'
+            id = Column(Integer, ForeignKey('b.id'), primary_key=True)
+
+    def test_ordered_b_d(self):
+        a_mapper = inspect(self.classes.A)
+        eq_(
+            a_mapper._mappers_from_spec(
+                    [self.classes.B, self.classes.D], None),
+            [a_mapper, inspect(self.classes.B), inspect(self.classes.D)]
+        )
+
+    def test_a(self):
+        a_mapper = inspect(self.classes.A)
+        eq_(
+            a_mapper._mappers_from_spec(
+                    [self.classes.A], None),
+            [a_mapper]
+        )
+
+    def test_b_d_selectable(self):
+        a_mapper = inspect(self.classes.A)
+        spec = [self.classes.D, self.classes.B]
+        eq_(
+            a_mapper._mappers_from_spec(
+                    spec,
+                    self.classes.B.__table__.join(self.classes.D.__table__)
+            ),
+            [inspect(self.classes.B), inspect(self.classes.D)]
+        )
+
+    def test_d_selectable(self):
+        a_mapper = inspect(self.classes.A)
+        spec = [self.classes.D]
+        eq_(
+            a_mapper._mappers_from_spec(
+                    spec,
+                    self.classes.B.__table__.join(self.classes.D.__table__)
+            ),
+            [inspect(self.classes.D)]
+        )
+
+    def test_reverse_d_b(self):
+        a_mapper = inspect(self.classes.A)
+        spec = [self.classes.D, self.classes.B]
+        eq_(
+            a_mapper._mappers_from_spec(
+                    spec, None),
+            [a_mapper, inspect(self.classes.B), inspect(self.classes.D)]
+        )
+        mappers, selectable = a_mapper._with_polymorphic_args(spec=spec)
+        self.assert_compile(selectable,
+            "a LEFT OUTER JOIN b ON a.id = b.id "
+            "LEFT OUTER JOIN d ON b.id = d.id")
+
+    def test_d_b_missing(self):
+        a_mapper = inspect(self.classes.A)
+        spec = [self.classes.D]
+        eq_(
+            a_mapper._mappers_from_spec(
+                    spec, None),
+            [a_mapper, inspect(self.classes.B), inspect(self.classes.D)]
+        )
+        mappers, selectable = a_mapper._with_polymorphic_args(spec=spec)
+        self.assert_compile(selectable,
+            "a LEFT OUTER JOIN b ON a.id = b.id "
+            "LEFT OUTER JOIN d ON b.id = d.id")
+
+    def test_d_c_b(self):
+        a_mapper = inspect(self.classes.A)
+        spec = [self.classes.D, self.classes.C, self.classes.B]
+        ms = a_mapper._mappers_from_spec(spec, None)
+
+        eq_(
+            ms[-1], inspect(self.classes.D)
+        )
+        eq_(ms[0], a_mapper)
+        eq_(
+            set(ms[1:3]), set(a_mapper._inheriting_mappers)
+        )
 
 class PolymorphicOnNotLocalTest(fixtures.MappedTest):
     @classmethod
@@ -325,6 +426,44 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
             [Child]
         )
 
+class SortOnlyOnImportantFKsTest(fixtures.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('a', metadata,
+                Column('id', Integer, primary_key=True,
+                                    test_needs_autoincrement=True),
+                Column('b_id', Integer,
+                        ForeignKey('b.id', use_alter=True, name='b'))
+            )
+        Table('b', metadata,
+            Column('id', Integer, ForeignKey('a.id'), primary_key=True)
+            )
+
+    @classmethod
+    def setup_classes(cls):
+        Base = declarative_base()
+
+        class A(Base):
+            __tablename__ = "a"
+
+            id = Column(Integer, primary_key=True,
+                                    test_needs_autoincrement=True)
+            b_id = Column(Integer, ForeignKey('b.id'))
+
+        class B(A):
+            __tablename__ = "b"
+
+            id = Column(Integer, ForeignKey('a.id'), primary_key=True)
+
+            __mapper_args__ = {'inherit_condition': id == A.id}
+
+        cls.classes.A = A
+        cls.classes.B = B
+
+    def test_flush(self):
+        s = Session(testing.db)
+        s.add(self.classes.B())
+        s.flush()
 
 class FalseDiscriminatorTest(fixtures.MappedTest):
     @classmethod
@@ -335,8 +474,10 @@ class FalseDiscriminatorTest(fixtures.MappedTest):
             Column('type', Boolean, nullable=False))
 
     def test_false_on_sub(self):
-        class Foo(object):pass
-        class Bar(Foo):pass
+        class Foo(object):
+            pass
+        class Bar(Foo):
+            pass
         mapper(Foo, t1, polymorphic_on=t1.c.type, polymorphic_identity=True)
         mapper(Bar, inherits=Foo, polymorphic_identity=False)
         sess = create_session()
@@ -414,11 +555,12 @@ class PolymorphicAttributeManagementTest(fixtures.MappedTest):
         Table('table_b', metadata,
            Column('id', Integer, ForeignKey('table_a.id'),
                                 primary_key=True),
-           Column('class_name', String(50))
+           Column('class_name', String(50)),
         )
         Table('table_c', metadata,
            Column('id', Integer, ForeignKey('table_b.id'),
-                                primary_key=True)
+                                primary_key=True),
+           Column('data', String(10))
         )
 
     @classmethod
@@ -433,6 +575,8 @@ class PolymorphicAttributeManagementTest(fixtures.MappedTest):
             pass
         class C(B):
             pass
+        class D(B):
+            pass
 
         mapper(A, table_a,
                         polymorphic_on=table_a.c.class_name,
@@ -442,6 +586,8 @@ class PolymorphicAttributeManagementTest(fixtures.MappedTest):
                         polymorphic_identity='b')
         mapper(C, table_c, inherits=B,
                         polymorphic_identity='c')
+        mapper(D, inherits=B,
+                        polymorphic_identity='d')
 
     def test_poly_configured_immediate(self):
         A, C, B = (self.classes.A,
@@ -471,16 +617,103 @@ class PolymorphicAttributeManagementTest(fixtures.MappedTest):
 
         assert isinstance(sess.query(A).first(), C)
 
-    def test_assignment(self):
-        C, B = self.classes.C, self.classes.B
+    def test_valid_assignment_upwards(self):
+        """test that we can assign 'd' to a B, since B/D
+        both involve the same set of tables.
+        """
+        D, B = self.classes.D, self.classes.B
+
+        sess = Session()
+        b1 = B()
+        b1.class_name = 'd'
+        sess.add(b1)
+        sess.commit()
+        sess.close()
+        assert isinstance(sess.query(B).first(), D)
+
+    def test_invalid_assignment_downwards(self):
+        """test that we warn on assign of 'b' to a C, since this adds
+        a row to the C table we'd never load.
+        """
+        C = self.classes.C
+
+        sess = Session()
+        c1 = C()
+        c1.class_name = 'b'
+        sess.add(c1)
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Flushing object %s with incompatible "
+            "polymorphic identity 'b'; the object may not "
+            "refresh and/or load correctly" % instance_str(c1),
+            sess.flush
+        )
+
+    def test_invalid_assignment_upwards(self):
+        """test that we warn on assign of 'c' to a B, since we will have a
+        "C" row that has no joined row, which will cause object
+        deleted errors.
+        """
+        B = self.classes.B
 
         sess = Session()
         b1 = B()
         b1.class_name = 'c'
         sess.add(b1)
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Flushing object %s with incompatible "
+            "polymorphic identity 'c'; the object may not "
+            "refresh and/or load correctly" % instance_str(b1),
+            sess.flush
+        )
+
+    def test_entirely_oob_assignment(self):
+        """test warn on an unknown polymorphic identity.
+        """
+        B = self.classes.B
+
+        sess = Session()
+        b1 = B()
+        b1.class_name = 'xyz'
+        sess.add(b1)
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Flushing object %s with incompatible "
+            "polymorphic identity 'xyz'; the object may not "
+            "refresh and/or load correctly" % instance_str(b1),
+            sess.flush
+        )
+
+    def test_not_set_on_upate(self):
+        C = self.classes.C
+
+        sess = Session()
+        c1 = C()
+        sess.add(c1)
         sess.commit()
-        sess.close()
-        assert isinstance(sess.query(B).first(), C)
+        sess.expire(c1)
+
+        c1.data = 'foo'
+        sess.flush()
+
+    def test_validate_on_upate(self):
+        C = self.classes.C
+
+        sess = Session()
+        c1 = C()
+        sess.add(c1)
+        sess.commit()
+        sess.expire(c1)
+
+        c1.class_name = 'b'
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Flushing object %s with incompatible "
+            "polymorphic identity 'b'; the object may not "
+            "refresh and/or load correctly" % instance_str(c1),
+            sess.flush
+        )
 
 class CascadeTest(fixtures.MappedTest):
     """that cascades on polymorphic relationships continue
@@ -863,7 +1096,7 @@ class FlushTest(fixtures.MappedTest):
 
         # create an Admin, and append a Role.  the dependency processors
         # corresponding to the "roles" attribute for the Admin mapper and the User mapper
-        # have to ensure that two dependency processors dont fire off and insert the
+        # have to ensure that two dependency processors don't fire off and insert the
         # many to many row twice.
         a = Admin()
         a.roles.append(adminrole)
@@ -915,6 +1148,73 @@ class FlushTest(fixtures.MappedTest):
         sess.flush()
         assert user_roles.count().scalar() == 1
 
+class JoinedNoFKSortingTest(fixtures.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table("a", metadata,
+                Column('id', Integer, primary_key=True,
+                    test_needs_autoincrement=True)
+            )
+        Table("b", metadata,
+                Column('id', Integer, primary_key=True)
+            )
+        Table("c", metadata,
+                Column('id', Integer, primary_key=True)
+            )
+
+    @classmethod
+    def setup_classes(cls):
+        class A(cls.Basic):
+            pass
+        class B(A):
+            pass
+        class C(A):
+            pass
+
+    @classmethod
+    def setup_mappers(cls):
+        A, B, C = cls.classes.A, cls.classes.B, cls.classes.C
+        mapper(A, cls.tables.a)
+        mapper(B, cls.tables.b, inherits=A,
+                    inherit_condition=cls.tables.a.c.id == cls.tables.b.c.id)
+        mapper(C, cls.tables.c, inherits=A,
+                    inherit_condition=cls.tables.a.c.id == cls.tables.c.c.id)
+
+    def test_ordering(self):
+        B, C = self.classes.B, self.classes.C
+        sess = Session()
+        sess.add_all([B(), C(), B(), C()])
+        self.assert_sql_execution(
+                testing.db,
+                sess.flush,
+                CompiledSQL(
+                    "INSERT INTO a () VALUES ()",
+                    {}
+                ),
+                CompiledSQL(
+                    "INSERT INTO a () VALUES ()",
+                    {}
+                ),
+                CompiledSQL(
+                    "INSERT INTO a () VALUES ()",
+                    {}
+                ),
+                CompiledSQL(
+                    "INSERT INTO a () VALUES ()",
+                    {}
+                ),
+                AllOf(
+                    CompiledSQL(
+                        "INSERT INTO b (id) VALUES (:id)",
+                        [{"id": 1}, {"id": 3}]
+                    ),
+                    CompiledSQL(
+                        "INSERT INTO c (id) VALUES (:id)",
+                        [{"id": 2}, {"id": 4}]
+                    )
+                )
+        )
+
 class VersioningTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -1430,6 +1730,53 @@ class OptimizedLoadTest(fixtures.MappedTest):
             Column('b', String(10))
         )
 
+    def test_no_optimize_on_map_to_join(self):
+        base, sub = self.tables.base, self.tables.sub
+
+        class Base(fixtures.ComparableEntity):
+            pass
+
+        class JoinBase(fixtures.ComparableEntity):
+            pass
+        class SubJoinBase(JoinBase):
+            pass
+
+        mapper(Base, base)
+        mapper(JoinBase, base.outerjoin(sub), properties=util.OrderedDict(
+                [('id', [base.c.id, sub.c.id]),
+                ('counter', [base.c.counter, sub.c.counter])])
+            )
+        mapper(SubJoinBase, inherits=JoinBase)
+
+        sess = Session()
+        sess.add(Base(data='data'))
+        sess.commit()
+
+        sjb = sess.query(SubJoinBase).one()
+        sjb_id = sjb.id
+        sess.expire(sjb)
+
+        # this should not use the optimized load,
+        # which assumes discrete tables
+        def go():
+            eq_(sjb.data, 'data')
+
+        self.assert_sql_execution(
+            testing.db,
+            go,
+            CompiledSQL(
+                "SELECT base.id AS base_id, sub.id AS sub_id, "
+                "base.counter AS base_counter, sub.counter AS sub_counter, "
+                "base.data AS base_data, "
+                "base.type AS base_type, sub.sub AS sub_sub, "
+                "sub.counter2 AS sub_counter2 FROM base "
+                "LEFT OUTER JOIN sub ON base.id = sub.id "
+                "WHERE base.id = :param_1",
+                {'param_1': sjb_id}
+            ),
+        )
+
+
     def test_optimized_passes(self):
         """"test that the 'optimized load' routine doesn't crash when
         a column in the join condition is not available."""
@@ -1471,7 +1818,7 @@ class OptimizedLoadTest(fixtures.MappedTest):
             pass
         mapper(Base, base, polymorphic_on=base.c.type, polymorphic_identity='base')
         mapper(Sub, sub, inherits=Base, polymorphic_identity='sub', properties={
-            'concat':column_property(sub.c.sub + "|" + sub.c.sub)
+            'concat': column_property(sub.c.sub + "|" + sub.c.sub)
         })
         sess = sessionmaker()()
         s1 = Sub(data='s1data', sub='s1sub')
@@ -1490,7 +1837,7 @@ class OptimizedLoadTest(fixtures.MappedTest):
             pass
         mapper(Base, base, polymorphic_on=base.c.type, polymorphic_identity='base')
         mapper(Sub, sub, inherits=Base, polymorphic_identity='sub', properties={
-            'concat':column_property(base.c.data + "|" + sub.c.sub)
+            'concat': column_property(base.c.data + "|" + sub.c.sub)
         })
         sess = sessionmaker()()
         s1 = Sub(data='s1data', sub='s1sub')
@@ -1603,7 +1950,7 @@ class OptimizedLoadTest(fixtures.MappedTest):
                                 ['counter2']) is None
 
         s1.id = 1
-        attributes.instance_state(s1).commit_all(s1.__dict__, None)
+        attributes.instance_state(s1)._commit_all(s1.__dict__, None)
         assert m._optimized_get_statement(attributes.instance_state(s1),
                                 ['counter2']) is not None
 
@@ -1660,6 +2007,66 @@ class OptimizedLoadTest(fixtures.MappedTest):
             ),
         )
 
+class TransientInheritingGCTest(fixtures.TestBase):
+    __requires__ = ('cpython',)
+
+    def _fixture(self):
+        Base = declarative_base()
+
+        class A(Base):
+            __tablename__ = 'a'
+            id = Column(Integer, primary_key=True,
+                                    test_needs_autoincrement=True)
+            data = Column(String(10))
+        self.A = A
+        return Base
+
+    def setUp(self):
+        self.Base = self._fixture()
+
+    def tearDown(self):
+        self.Base.metadata.drop_all(testing.db)
+        #clear_mappers()
+        self.Base = None
+
+    def _do_test(self, go):
+        B = go()
+        self.Base.metadata.create_all(testing.db)
+        sess = Session(testing.db)
+        sess.add(B(data='some b'))
+        sess.commit()
+
+        b1 = sess.query(B).one()
+        assert isinstance(b1, B)
+        sess.close()
+        del sess
+        del b1
+        del B
+
+        gc_collect()
+
+        eq_(
+            len(self.A.__subclasses__()),
+            0)
+
+    def test_single(self):
+        def go():
+            class B(self.A):
+                pass
+            return B
+        self._do_test(go)
+
+    @testing.fails_if(lambda: True,
+                "not supported for joined inh right now.")
+    def test_joined(self):
+        def go():
+            class B(self.A):
+                __tablename__ = 'b'
+                id = Column(Integer, ForeignKey('a.id'),
+                        primary_key=True)
+            return B
+        self._do_test(go)
+
 class NoPKOnSubTableWarningTest(fixtures.TestBase):
 
     def _fixture(self):
