@@ -9,16 +9,16 @@ UPDATE in the database.
 
 """
 
-from test.lib.testing import assert_raises, assert_raises_message
+from sqlalchemy.testing import assert_raises, assert_raises_message
 from sqlalchemy import Integer, String, ForeignKey, Sequence, exc as sa_exc
-from test.lib.schema import Table
-from test.lib.schema import Column
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
     class_mapper, backref, sessionmaker, Session
 from sqlalchemy.orm import attributes, exc as orm_exc
-from test.lib import testing
-from test.lib.testing import eq_
-from test.lib import fixtures
+from sqlalchemy import testing
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 
 class O2MCollectionTest(_fixtures.FixtureTest):
@@ -121,7 +121,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         # backref fires
         assert a1.user is u2
 
-        # everything expires, no changes in 
+        # everything expires, no changes in
         # u1.addresses, so all is fine
         sess.commit()
         assert a1 not in u1.addresses
@@ -143,7 +143,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         u1.addresses
 
         # direct set - the "old" is "fetched",
-        # but only from the local session - not the 
+        # but only from the local session - not the
         # database, due to the PASSIVE_NO_FETCH flag.
         # this is a more fine grained behavior introduced
         # in 0.6
@@ -207,7 +207,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         sess.add_all([u1, u2, a1])
         sess.commit()
 
-        # direct set - the fetching of the 
+        # direct set - the fetching of the
         # "old" u1 here allows the backref
         # to remove it from the addresses collection
         a1.user = u2
@@ -230,7 +230,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         # u1.addresses is loaded
         u1.addresses
 
-        # direct set - the fetching of the 
+        # direct set - the fetching of the
         # "old" u1 here allows the backref
         # to remove it from the addresses collection
         a1.user = u2
@@ -395,7 +395,7 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
         # backref fires
         assert u1.address is a2
 
-        # didnt work this way tho
+        # didn't work this way tho
         assert a1.user is u1
 
         # moves appropriately after commit
@@ -455,8 +455,8 @@ class O2OScalarOrphanTest(_fixtures.FixtureTest):
 
         mapper(Address, addresses)
         mapper(User, users, properties = {
-            'address':relationship(Address, uselist=False, 
-                backref=backref('user', single_parent=True, 
+            'address':relationship(Address, uselist=False,
+                backref=backref('user', single_parent=True,
                                     cascade="all, delete-orphan"))
         })
 
@@ -491,7 +491,7 @@ class M2MCollectionMoveTest(_fixtures.FixtureTest):
                                 cls.classes.Item)
 
         mapper(Item, items, properties={
-            'keywords':relationship(Keyword, secondary=item_keywords, 
+            'keywords':relationship(Keyword, secondary=item_keywords,
                                     backref='items')
         })
         mapper(Keyword, keywords)
@@ -551,7 +551,7 @@ class M2MCollectionMoveTest(_fixtures.FixtureTest):
         # list is still here.
         eq_(
             set(attributes.instance_state(i1).
-                pending['keywords'].added_items),
+                _pending_mutations['keywords'].added_items),
             set([k2])
         )
         # because autoflush is off, k2 is still
@@ -564,7 +564,7 @@ class M2MCollectionMoveTest(_fixtures.FixtureTest):
         # the pending collection was removed
         assert 'keywords' not in attributes.\
                                 instance_state(i1).\
-                                pending
+                                _pending_mutations
 
     def test_duplicate_adds(self):
         Item, Keyword = (self.classes.Item, self.classes.Keyword)
@@ -603,8 +603,8 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
                                 cls.classes.Item)
 
         mapper(Item, items, properties={
-            'keyword':relationship(Keyword, secondary=item_keywords, 
-                                    uselist=False, 
+            'keyword':relationship(Keyword, secondary=item_keywords,
+                                    uselist=False,
                                     backref=backref("item", uselist=False))
         })
         mapper(Keyword, keywords)
@@ -673,3 +673,65 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
         sess.commit()
         assert i1.keyword is None
         assert i2.keyword is k1
+
+class O2MStaleBackrefTest(_fixtures.FixtureTest):
+    run_inserts = None
+
+    @classmethod
+    def setup_mappers(cls):
+        Address, addresses, users, User = (cls.classes.Address,
+                                cls.tables.addresses,
+                                cls.tables.users,
+                                cls.classes.User)
+
+        mapper(Address, addresses)
+        mapper(User, users, properties = dict(
+            addresses = relationship(Address, backref="user"),
+        ))
+
+
+    def test_backref_pop_m2o(self):
+        User, Address = self.classes.User, self.classes.Address
+
+        u1 = User()
+        u2 = User()
+        a1 = Address()
+        u1.addresses.append(a1)
+        u2.addresses.append(a1)
+
+        # events haven't updated
+        # u1.addresses here.
+        u1.addresses.remove(a1)
+
+        assert a1.user is u2
+        assert a1 in u2.addresses
+
+class M2MStaleBackrefTest(_fixtures.FixtureTest):
+    run_inserts = None
+
+    @classmethod
+    def setup_mappers(cls):
+        keywords, items, item_keywords, Keyword, Item = (cls.tables.keywords,
+                                cls.tables.items,
+                                cls.tables.item_keywords,
+                                cls.classes.Keyword,
+                                cls.classes.Item)
+
+        mapper(Item, items, properties={
+            'keywords':relationship(Keyword, secondary=item_keywords,
+                                    backref='items')
+        })
+        mapper(Keyword, keywords)
+
+    def test_backref_pop_m2m(self):
+        Keyword, Item = self.classes.Keyword, self.classes.Item
+
+        k1 = Keyword()
+        k2 = Keyword()
+        i1 = Item()
+        k1.items.append(i1)
+        k2.items.append(i1)
+        k2.items.append(i1)
+        i1.keywords = []
+        k2.items.remove(i1)
+        assert len(k2.items) == 0
