@@ -5,17 +5,17 @@ T1<->T2, with o2m or m2o between them, and a third T3 with o2m/m2o to one/both
 T1/T2.
 
 """
-from sqlalchemy.test import testing
+from sqlalchemy import testing
 from sqlalchemy import Integer, String, ForeignKey
-from sqlalchemy.test.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, backref, \
                             create_session, sessionmaker
-from sqlalchemy.test.testing import eq_
-from sqlalchemy.test.assertsql import RegexSQL, ExactSQL, CompiledSQL, AllOf
-from test.orm import _base
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing.assertsql import RegexSQL, ExactSQL, CompiledSQL, AllOf
+from sqlalchemy.testing import fixtures
 
 
-class SelfReferentialTest(_base.MappedTest):
+class SelfReferentialTest(fixtures.MappedTest):
     """A self-referential mapper with an additional list of child objects."""
 
     @classmethod
@@ -33,16 +33,17 @@ class SelfReferentialTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class C1(_base.BasicEntity):
+        class C1(cls.Basic):
             def __init__(self, data=None):
                 self.data = data
 
-        class C2(_base.BasicEntity):
+        class C2(cls.Basic):
             def __init__(self, data=None):
                 self.data = data
 
-    @testing.resolve_artifact_names
     def test_single(self):
+        C1, t1 = self.classes.C1, self.tables.t1
+
         mapper(C1, t1, properties = {
             'c1s':relationship(C1, cascade="all"),
             'parent':relationship(C1,
@@ -59,15 +60,17 @@ class SelfReferentialTest(_base.MappedTest):
         sess.delete(a)
         sess.flush()
 
-    @testing.resolve_artifact_names
     def test_many_to_one_only(self):
         """
 
         test that the circular dependency sort can assemble a many-to-one
         dependency processor when only the object on the "many" side is
-        actually in the list of modified objects.  
+        actually in the list of modified objects.
 
         """
+
+        C1, t1 = self.classes.C1, self.tables.t1
+
         mapper(C1, t1, properties={
             'parent':relationship(C1,
                               primaryjoin=t1.c.parent_c1 == t1.c.c1,
@@ -86,8 +89,12 @@ class SelfReferentialTest(_base.MappedTest):
         sess.flush()
         assert c2.parent_c1==c1.c1
 
-    @testing.resolve_artifact_names
     def test_cycle(self):
+        C2, C1, t2, t1 = (self.classes.C2,
+                                self.classes.C1,
+                                self.tables.t2,
+                                self.tables.t1)
+
         mapper(C1, t1, properties = {
             'c1s' : relationship(C1, cascade="all"),
             'c2s' : relationship(mapper(C2, t2), cascade="all, delete-orphan")})
@@ -106,12 +113,13 @@ class SelfReferentialTest(_base.MappedTest):
         sess.delete(a)
         sess.flush()
 
-    @testing.resolve_artifact_names
     def test_setnull_ondelete(self):
+        C1, t1 = self.classes.C1, self.tables.t1
+
         mapper(C1, t1, properties={
             'children':relationship(C1)
         })
-        
+
         sess = create_session()
         c1 = C1()
         c2 = C1()
@@ -119,15 +127,15 @@ class SelfReferentialTest(_base.MappedTest):
         sess.add(c1)
         sess.flush()
         assert c2.parent_c1 == c1.c1
-        
+
         sess.delete(c1)
         sess.flush()
         assert c2.parent_c1 is None
-        
+
         sess.expire_all()
         assert c2.parent_c1 is None
-        
-class SelfReferentialNoPKTest(_base.MappedTest):
+
+class SelfReferentialNoPKTest(fixtures.MappedTest):
     """A self-referential relationship that joins on a column other than the primary key column"""
 
     @classmethod
@@ -140,21 +148,23 @@ class SelfReferentialNoPKTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class TT(_base.BasicEntity):
+        class TT(cls.Basic):
             def __init__(self):
                 self.uuid = hex(id(self))
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        item, TT = cls.tables.item, cls.classes.TT
+
         mapper(TT, item, properties={
             'children': relationship(
                 TT,
                 remote_side=[item.c.parent_uuid],
                 backref=backref('parent', remote_side=[item.c.uuid]))})
 
-    @testing.resolve_artifact_names
     def test_basic(self):
+        TT = self.classes.TT
+
         t1 = TT()
         t1.children.append(TT())
         t1.children.append(TT())
@@ -166,8 +176,9 @@ class SelfReferentialNoPKTest(_base.MappedTest):
         t = s.query(TT).filter_by(id=t1.id).one()
         eq_(t.children[0].parent_uuid, t1.uuid)
 
-    @testing.resolve_artifact_names
     def test_lazy_clause(self):
+        TT = self.classes.TT
+
         s = create_session()
         t1 = TT()
         t2 = TT()
@@ -181,7 +192,7 @@ class SelfReferentialNoPKTest(_base.MappedTest):
         eq_(t.parent.uuid, t1.uuid)
 
 
-class InheritTestOne(_base.MappedTest):
+class InheritTestOne(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table("parent", metadata,
@@ -201,7 +212,7 @@ class InheritTestOne(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Parent(_base.BasicEntity):
+        class Parent(cls.Basic):
             pass
 
         class Child1(Parent):
@@ -211,18 +222,26 @@ class InheritTestOne(_base.MappedTest):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        child1, child2, parent, Parent, Child1, Child2 = (cls.tables.child1,
+                                cls.tables.child2,
+                                cls.tables.parent,
+                                cls.classes.Parent,
+                                cls.classes.Child1,
+                                cls.classes.Child2)
+
         mapper(Parent, parent)
         mapper(Child1, child1, inherits=Parent)
         mapper(Child2, child2, inherits=Parent, properties=dict(
             child1=relationship(Child1,
                             primaryjoin=child2.c.child1_id == child1.c.id)))
 
-    @testing.resolve_artifact_names
     def test_many_to_one_only(self):
         """test similar to SelfReferentialTest.testmanytooneonly"""
 
+        Child1, Child2 = self.classes.Child1, self.classes.Child2
+
+
         session = create_session()
 
         c1 = Child1()
@@ -243,7 +262,7 @@ class InheritTestOne(_base.MappedTest):
         session.flush()
 
 
-class InheritTestTwo(_base.MappedTest):
+class InheritTestTwo(fixtures.MappedTest):
     """
 
     The fix in BiDirectionalManyToOneTest raised this issue, regarding the
@@ -271,17 +290,23 @@ class InheritTestTwo(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class A(_base.BasicEntity):
+        class A(cls.Basic):
             pass
 
         class B(A):
             pass
 
-        class C(_base.BasicEntity):
+        class C(cls.Basic):
             pass
 
-    @testing.resolve_artifact_names
     def test_flush(self):
+        a, A, c, b, C, B = (self.tables.a,
+                                self.classes.A,
+                                self.tables.c,
+                                self.tables.b,
+                                self.classes.C,
+                                self.classes.B)
+
         mapper(A, a, properties={
             'cs':relationship(C, primaryjoin=a.c.cid==c.c.id)})
 
@@ -298,9 +323,9 @@ class InheritTestTwo(_base.MappedTest):
         sess.flush()
 
 
-class BiDirectionalManyToOneTest(_base.MappedTest):
+class BiDirectionalManyToOneTest(fixtures.MappedTest):
     run_define_tables = 'each'
-    
+
     @classmethod
     def define_tables(cls, metadata):
         Table('t1', metadata,
@@ -320,16 +345,22 @@ class BiDirectionalManyToOneTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class T1(_base.BasicEntity):
+        class T1(cls.Basic):
             pass
-        class T2(_base.BasicEntity):
+        class T2(cls.Basic):
             pass
-        class T3(_base.BasicEntity):
+        class T3(cls.Basic):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        t2, T2, T3, t1, t3, T1 = (cls.tables.t2,
+                                cls.classes.T2,
+                                cls.classes.T3,
+                                cls.tables.t1,
+                                cls.tables.t3,
+                                cls.classes.T1)
+
         mapper(T1, t1, properties={
             't2':relationship(T2, primaryjoin=t1.c.t2id == t2.c.id)})
         mapper(T2, t2, properties={
@@ -338,8 +369,11 @@ class BiDirectionalManyToOneTest(_base.MappedTest):
             't1':relationship(T1),
             't2':relationship(T2)})
 
-    @testing.resolve_artifact_names
     def test_reflush(self):
+        T2, T3, T1 = (self.classes.T2,
+                                self.classes.T3,
+                                self.classes.T1)
+
         o1 = T1()
         o1.t2 = T2()
         sess = create_session()
@@ -349,7 +383,7 @@ class BiDirectionalManyToOneTest(_base.MappedTest):
         # the bug here is that the dependency sort comes up with T1/T2 in a
         # cycle, but there are no T1/T2 objects to be saved.  therefore no
         # "cyclical subtree" gets generated, and one or the other of T1/T2
-        # gets lost, and processors on T3 dont fire off.  the test will then
+        # gets lost, and processors on T3 don't fire off.  the test will then
         # fail because the FK's on T3 are not nullable.
         o3 = T3()
         o3.t1 = o1
@@ -357,9 +391,13 @@ class BiDirectionalManyToOneTest(_base.MappedTest):
         sess.add(o3)
         sess.flush()
 
-    @testing.resolve_artifact_names
     def test_reflush_2(self):
         """A variant on test_reflush()"""
+
+        T2, T3, T1 = (self.classes.T2,
+                                self.classes.T3,
+                                self.classes.T1)
+
         o1 = T1()
         o1.t2 = T2()
         sess = create_session()
@@ -386,7 +424,7 @@ class BiDirectionalManyToOneTest(_base.MappedTest):
         sess.flush()
 
 
-class BiDirectionalOneToManyTest(_base.MappedTest):
+class BiDirectionalOneToManyTest(fixtures.MappedTest):
     """tests two mappers with a one-to-many relationship to each other."""
 
     run_define_tables = 'each'
@@ -404,14 +442,18 @@ class BiDirectionalOneToManyTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class C1(_base.BasicEntity):
+        class C1(cls.Basic):
             pass
 
-        class C2(_base.BasicEntity):
+        class C2(cls.Basic):
             pass
 
-    @testing.resolve_artifact_names
     def test_cycle(self):
+        C2, C1, t2, t1 = (self.classes.C2,
+                                self.classes.C1,
+                                self.tables.t2,
+                                self.tables.t1)
+
         mapper(C2, t2, properties={
             'c1s': relationship(C1,
                             primaryjoin=t2.c.c1 == t1.c.c2,
@@ -435,8 +477,8 @@ class BiDirectionalOneToManyTest(_base.MappedTest):
         sess.flush()
 
 
-class BiDirectionalOneToManyTest2(_base.MappedTest):
-    """Two mappers with a one-to-many relationship to each other, 
+class BiDirectionalOneToManyTest2(fixtures.MappedTest):
+    """Two mappers with a one-to-many relationship to each other,
     with a second one-to-many on one of the mappers"""
 
     run_define_tables = 'each'
@@ -462,18 +504,24 @@ class BiDirectionalOneToManyTest2(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class C1(_base.BasicEntity):
+        class C1(cls.Basic):
             pass
 
-        class C2(_base.BasicEntity):
+        class C2(cls.Basic):
             pass
 
-        class C1Data(_base.BasicEntity):
+        class C1Data(cls.Basic):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        t2, t1, C1Data, t1_data, C2, C1 = (cls.tables.t2,
+                                cls.tables.t1,
+                                cls.classes.C1Data,
+                                cls.tables.t1_data,
+                                cls.classes.C2,
+                                cls.classes.C1)
+
         mapper(C2, t2, properties={
             'c1s': relationship(C1,
                             primaryjoin=t2.c.c1 == t1.c.c2,
@@ -484,8 +532,11 @@ class BiDirectionalOneToManyTest2(_base.MappedTest):
                              uselist=True),
             'data': relationship(mapper(C1Data, t1_data))})
 
-    @testing.resolve_artifact_names
     def test_cycle(self):
+        C2, C1, C1Data = (self.classes.C2,
+                                self.classes.C1,
+                                self.classes.C1Data)
+
         a = C1()
         b = C2()
         c = C1()
@@ -506,7 +557,7 @@ class BiDirectionalOneToManyTest2(_base.MappedTest):
         sess.delete(c)
         sess.flush()
 
-class OneToManyManyToOneTest(_base.MappedTest):
+class OneToManyManyToOneTest(fixtures.MappedTest):
     """
 
     Tests two mappers, one has a one-to-many on the other mapper, the other
@@ -517,7 +568,7 @@ class OneToManyManyToOneTest(_base.MappedTest):
 
     """
     run_define_tables = 'each'
-    
+
     @classmethod
     def define_tables(cls, metadata):
         Table('ball', metadata,
@@ -533,20 +584,25 @@ class OneToManyManyToOneTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Person(_base.ComparableEntity):
+        class Person(cls.Basic):
             pass
 
-        class Ball(_base.ComparableEntity):
+        class Ball(cls.Basic):
             pass
 
-    @testing.resolve_artifact_names
     def test_cycle(self):
         """
-        This test has a peculiar aspect in that it doesnt create as many
+        This test has a peculiar aspect in that it doesn't create as many
         dependent relationships as the other tests, and revealed a small
         glitch in the circular dependency sorting.
 
         """
+
+        person, ball, Ball, Person = (self.tables.person,
+                                self.tables.ball,
+                                self.classes.Ball,
+                                self.classes.Person)
+
         mapper(Ball, ball)
         mapper(Person, person, properties=dict(
             balls=relationship(Ball,
@@ -563,9 +619,14 @@ class OneToManyManyToOneTest(_base.MappedTest):
         sess.add(p)
         sess.flush()
 
-    @testing.resolve_artifact_names
     def test_post_update_m2o(self):
         """A cycle between two rows, with a post_update on the many-to-one"""
+
+        person, ball, Ball, Person = (self.tables.person,
+                                self.tables.ball,
+                                self.classes.Ball,
+                                self.classes.Person)
+
         mapper(Ball, ball)
         mapper(Person, person, properties=dict(
             balls=relationship(Ball,
@@ -606,8 +667,8 @@ class OneToManyManyToOneTest(_base.MappedTest):
         sess.delete(p)
 
         self.assert_sql_execution(
-            testing.db, 
-            sess.flush, 
+            testing.db,
+            sess.flush,
             ExactSQL("UPDATE person SET favorite_ball_id=:favorite_ball_id "
                 "WHERE person.id = :person_id",
                 lambda ctx: {'person_id': p.id, 'favorite_ball_id': None}),
@@ -615,10 +676,15 @@ class OneToManyManyToOneTest(_base.MappedTest):
             ExactSQL("DELETE FROM person WHERE person.id = :id", lambda ctx:[{'id': p.id}])
         )
 
-    @testing.resolve_artifact_names
     def test_post_update_backref(self):
         """test bidirectional post_update."""
-        
+
+        person, ball, Ball, Person = (self.tables.person,
+                                self.tables.ball,
+                                self.classes.Ball,
+                                self.classes.Person)
+
+
         mapper(Ball, ball)
         mapper(Person, person, properties=dict(
             balls=relationship(Ball,
@@ -629,20 +695,20 @@ class OneToManyManyToOneTest(_base.MappedTest):
            favorite=relationship(Ball,
                              primaryjoin=person.c.favorite_ball_id == ball.c.id,
                              remote_side=person.c.favorite_ball_id)
-            
+
             ))
-        
+
         sess = sessionmaker()()
         p1 = Person(data='p1')
         p2 = Person(data='p2')
         p3 = Person(data='p3')
-        
+
         b1 = Ball(data='b1')
-        
+
         b1.person = p1
         sess.add_all([p1, p2, p3])
         sess.commit()
-        
+
         # switch here.  the post_update
         # on ball.person can't get tripped up
         # by the fact that there's a "reverse" prop.
@@ -652,19 +718,24 @@ class OneToManyManyToOneTest(_base.MappedTest):
             p2, b1.person
         )
 
-        # do it the other way 
+        # do it the other way
         p3.balls.append(b1)
         sess.commit()
         eq_(
             p3, b1.person
         )
-        
 
 
-    @testing.resolve_artifact_names
+
     def test_post_update_o2m(self):
         """A cycle between two rows, with a post_update on the one-to-many"""
 
+        person, ball, Ball, Person = (self.tables.person,
+                                self.tables.ball,
+                                self.classes.Ball,
+                                self.classes.Person)
+
+
         mapper(Ball, ball)
         mapper(Person, person, properties=dict(
             balls=relationship(Ball,
@@ -713,44 +784,30 @@ class OneToManyManyToOneTest(_base.MappedTest):
              "VALUES (:favorite_ball_id, :data)",
              lambda ctx:{'favorite_ball_id':b.id, 'data':'some data'}),
 
-            AllOf(
-            CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id':p.id,'ball_id':b.id}),
-
-            CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id':p.id,'ball_id':b2.id}),
-
-            CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id':p.id,'ball_id':b3.id}),
-
-            CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id':p.id,'ball_id':b4.id})
+             CompiledSQL("UPDATE ball SET person_id=:person_id "
+              "WHERE ball.id = :ball_id",
+              lambda ctx:[
+                {'person_id':p.id,'ball_id':b.id},
+                {'person_id':p.id,'ball_id':b2.id},
+                {'person_id':p.id,'ball_id':b3.id},
+                {'person_id':p.id,'ball_id':b4.id}
+                ]
             ),
+
         )
-        
+
         sess.delete(p)
-        
-        self.assert_sql_execution(testing.db, sess.flush, 
-            AllOf(CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id': None, 'ball_id': b.id}),
 
+        self.assert_sql_execution(testing.db, sess.flush,
             CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id': None, 'ball_id': b2.id}),
-
-            CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id': None, 'ball_id': b3.id}),
-
-            CompiledSQL("UPDATE ball SET person_id=:person_id "
-             "WHERE ball.id = :ball_id",
-             lambda ctx:{'person_id': None, 'ball_id': b4.id})),
-
+                "WHERE ball.id = :ball_id",
+                lambda ctx:[
+                    {'person_id': None, 'ball_id': b.id},
+                    {'person_id': None, 'ball_id': b2.id},
+                    {'person_id': None, 'ball_id': b3.id},
+                    {'person_id': None, 'ball_id': b4.id}
+                ]
+            ),
             CompiledSQL("DELETE FROM person WHERE person.id = :id",
              lambda ctx:[{'id':p.id}]),
 
@@ -762,8 +819,11 @@ class OneToManyManyToOneTest(_base.MappedTest):
         )
 
 
-class SelfReferentialPostUpdateTest(_base.MappedTest):
-    """Post_update on a single self-referential mapper"""
+class SelfReferentialPostUpdateTest(fixtures.MappedTest):
+    """Post_update on a single self-referential mapper.
+
+
+    """
 
     @classmethod
     def define_tables(cls, metadata):
@@ -780,12 +840,11 @@ class SelfReferentialPostUpdateTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Node(_base.BasicEntity):
+        class Node(cls.Basic):
             def __init__(self, path=''):
                 self.path = path
 
-    @testing.resolve_artifact_names
-    def test_basic(self):
+    def test_one(self):
         """Post_update only fires off when needed.
 
         This test case used to produce many superfluous update statements,
@@ -793,11 +852,13 @@ class SelfReferentialPostUpdateTest(_base.MappedTest):
 
         """
 
+        node, Node = self.tables.node, self.classes.Node
+
+
         mapper(Node, node, properties={
             'children': relationship(
                 Node,
                 primaryjoin=node.c.id==node.c.parent_id,
-                lazy='select',
                 cascade="all",
                 backref=backref("parent", remote_side=node.c.id)
             ),
@@ -805,13 +866,11 @@ class SelfReferentialPostUpdateTest(_base.MappedTest):
                 Node,
                 primaryjoin=node.c.prev_sibling_id==node.c.id,
                 remote_side=node.c.id,
-                lazy='select',
                 uselist=False),
             'next_sibling': relationship(
                 Node,
                 primaryjoin=node.c.next_sibling_id==node.c.id,
                 remote_side=node.c.id,
-                lazy='select',
                 uselist=False,
                 post_update=True)})
 
@@ -849,10 +908,11 @@ class SelfReferentialPostUpdateTest(_base.MappedTest):
         session.flush()
 
         remove_child(root, cats)
+
         # pre-trigger lazy loader on 'cats' to make the test easier
         cats.children
         self.assert_sql_execution(
-            testing.db, 
+            testing.db,
             session.flush,
             AllOf(
             CompiledSQL("UPDATE node SET prev_sibling_id=:prev_sibling_id "
@@ -867,22 +927,22 @@ class SelfReferentialPostUpdateTest(_base.MappedTest):
              "WHERE node.id = :node_id",
              lambda ctx:{'next_sibling_id':None, 'node_id':cats.id}),
             ),
-             
+
             CompiledSQL("DELETE FROM node WHERE node.id = :id",
              lambda ctx:[{'id':cats.id}])
         )
 
         session.delete(root)
+
         self.assert_sql_execution(
-            testing.db, 
+            testing.db,
             session.flush,
-            AllOf(
-                CompiledSQL("UPDATE node SET next_sibling_id=:next_sibling_id "
-                            "WHERE node.id = :node_id", 
-                            lambda ctx:{'next_sibling_id':None, 'node_id':about.id}),
-                CompiledSQL("UPDATE node SET next_sibling_id=:next_sibling_id "
-                            "WHERE node.id = :node_id",
-                            lambda ctx:{'node_id':stories.id, 'next_sibling_id':None})
+            CompiledSQL("UPDATE node SET next_sibling_id=:next_sibling_id "
+                "WHERE node.id = :node_id",
+                lambda ctx: [
+                            {'node_id': about.id, 'next_sibling_id': None},
+                            {'node_id': stories.id, 'next_sibling_id': None}
+                        ]
             ),
             AllOf(
                 CompiledSQL("DELETE FROM node WHERE node.id = :id",
@@ -909,7 +969,7 @@ class SelfReferentialPostUpdateTest(_base.MappedTest):
         cats.prev_sibling = None
         session.flush()
 
-class SelfReferentialPostUpdateTest2(_base.MappedTest):
+class SelfReferentialPostUpdateTest2(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
@@ -920,11 +980,10 @@ class SelfReferentialPostUpdateTest2(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class A(_base.BasicEntity):
+        class A(cls.Basic):
             pass
 
-    @testing.resolve_artifact_names
-    def test_basic(self):
+    def test_one(self):
         """
         Test that post_update remembers to be involved in update operations as
         well, since it replaces the normal dependency processing completely
@@ -932,6 +991,9 @@ class SelfReferentialPostUpdateTest2(_base.MappedTest):
 
         """
 
+        A, a_table = self.classes.A, self.tables.a_table
+
+
         mapper(A, a_table, properties={
             'foo': relationship(A,
                             remote_side=[a_table.c.id],
@@ -956,3 +1018,185 @@ class SelfReferentialPostUpdateTest2(_base.MappedTest):
         assert f2.foo is f1
 
 
+class SelfReferentialPostUpdateTest3(fixtures.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('parent', metadata,
+              Column('id', Integer, primary_key=True,
+                     test_needs_autoincrement=True),
+              Column('name', String(50), nullable=False),
+              Column('child_id', Integer,
+                     ForeignKey('child.id', use_alter=True, name='c1'), nullable=True))
+
+        Table('child', metadata,
+           Column('id', Integer, primary_key=True,
+                  test_needs_autoincrement=True),
+           Column('name', String(50), nullable=False),
+           Column('child_id', Integer,
+                  ForeignKey('child.id')),
+           Column('parent_id', Integer,
+                  ForeignKey('parent.id'), nullable=True))
+
+    @classmethod
+    def setup_classes(cls):
+        class Parent(cls.Basic):
+            def __init__(self, name=''):
+                self.name = name
+
+        class Child(cls.Basic):
+            def __init__(self, name=''):
+                self.name = name
+
+    def test_one(self):
+        Child, Parent, parent, child = (self.classes.Child,
+                                self.classes.Parent,
+                                self.tables.parent,
+                                self.tables.child)
+
+        mapper(Parent, parent, properties={
+            'children':relationship(Child, primaryjoin=parent.c.id==child.c.parent_id),
+            'child':relationship(Child, primaryjoin=parent.c.child_id==child.c.id, post_update=True)
+        })
+        mapper(Child, child, properties={
+            'parent':relationship(Child, remote_side=child.c.id)
+        })
+
+        session = create_session()
+        p1 = Parent('p1')
+        c1 = Child('c1')
+        c2 = Child('c2')
+        p1.children =[c1, c2]
+        c2.parent = c1
+        p1.child = c2
+
+        session.add_all([p1, c1, c2])
+        session.flush()
+
+        p2 = Parent('p2')
+        c3 = Child('c3')
+        p2.children = [c3]
+        p2.child = c3
+        session.add(p2)
+
+        session.delete(c2)
+        p1.children.remove(c2)
+        p1.child = None
+        session.flush()
+
+        p2.child = None
+        session.flush()
+
+class PostUpdateBatchingTest(fixtures.MappedTest):
+    """test that lots of post update cols batch together into a single UPDATE."""
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('parent', metadata,
+              Column('id', Integer, primary_key=True,
+                     test_needs_autoincrement=True),
+              Column('name', String(50), nullable=False),
+              Column('c1_id', Integer,
+                     ForeignKey('child1.id', use_alter=True, name='c1'), nullable=True),
+              Column('c2_id', Integer,
+                    ForeignKey('child2.id', use_alter=True, name='c2'), nullable=True),
+              Column('c3_id', Integer,
+                       ForeignKey('child3.id', use_alter=True, name='c3'), nullable=True)
+            )
+
+        Table('child1', metadata,
+           Column('id', Integer, primary_key=True,
+                  test_needs_autoincrement=True),
+           Column('name', String(50), nullable=False),
+           Column('parent_id', Integer,
+                  ForeignKey('parent.id'), nullable=False))
+
+        Table('child2', metadata,
+             Column('id', Integer, primary_key=True,
+                    test_needs_autoincrement=True),
+             Column('name', String(50), nullable=False),
+             Column('parent_id', Integer,
+                    ForeignKey('parent.id'), nullable=False))
+
+        Table('child3', metadata,
+           Column('id', Integer, primary_key=True,
+                  test_needs_autoincrement=True),
+           Column('name', String(50), nullable=False),
+           Column('parent_id', Integer,
+                  ForeignKey('parent.id'), nullable=False))
+
+    @classmethod
+    def setup_classes(cls):
+        class Parent(cls.Basic):
+            def __init__(self, name=''):
+                self.name = name
+        class Child1(cls.Basic):
+            def __init__(self, name=''):
+                self.name = name
+        class Child2(cls.Basic):
+            def __init__(self, name=''):
+                self.name = name
+        class Child3(cls.Basic):
+            def __init__(self, name=''):
+                self.name = name
+
+    def test_one(self):
+        child1, child2, child3, Parent, parent, Child1, Child2, Child3 = (self.tables.child1,
+                                self.tables.child2,
+                                self.tables.child3,
+                                self.classes.Parent,
+                                self.tables.parent,
+                                self.classes.Child1,
+                                self.classes.Child2,
+                                self.classes.Child3)
+
+        mapper(Parent, parent, properties={
+            'c1s':relationship(Child1, primaryjoin=child1.c.parent_id==parent.c.id),
+            'c2s':relationship(Child2, primaryjoin=child2.c.parent_id==parent.c.id),
+            'c3s':relationship(Child3, primaryjoin=child3.c.parent_id==parent.c.id),
+
+            'c1':relationship(Child1, primaryjoin=child1.c.id==parent.c.c1_id, post_update=True),
+            'c2':relationship(Child2, primaryjoin=child2.c.id==parent.c.c2_id, post_update=True),
+            'c3':relationship(Child3, primaryjoin=child3.c.id==parent.c.c3_id, post_update=True),
+        })
+        mapper(Child1, child1)
+        mapper(Child2, child2)
+        mapper(Child3, child3)
+
+        sess = create_session()
+
+        p1 = Parent('p1')
+        c11, c12, c13 = Child1('c1'), Child1('c2'), Child1('c3')
+        c21, c22, c23 = Child2('c1'), Child2('c2'), Child2('c3')
+        c31, c32, c33 = Child3('c1'), Child3('c2'), Child3('c3')
+
+        p1.c1s = [c11, c12, c13]
+        p1.c2s = [c21, c22, c23]
+        p1.c3s = [c31, c32, c33]
+        sess.add(p1)
+        sess.flush()
+
+        p1.c1 = c12
+        p1.c2 = c23
+        p1.c3 = c31
+
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "UPDATE parent SET c1_id=:c1_id, c2_id=:c2_id, "
+                "c3_id=:c3_id WHERE parent.id = :parent_id",
+                lambda ctx: {'c2_id': c23.id, 'parent_id': p1.id, 'c1_id': c12.id, 'c3_id': c31.id}
+            )
+        )
+
+        p1.c1 = p1.c2 = p1.c3 = None
+
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "UPDATE parent SET c1_id=:c1_id, c2_id=:c2_id, "
+                "c3_id=:c3_id WHERE parent.id = :parent_id",
+                lambda ctx: {'c2_id': None, 'parent_id': p1.id, 'c1_id': None, 'c3_id': None}
+            )
+        )
