@@ -1,16 +1,16 @@
 """Attribute/instance expiration, deferral of attributes, etc."""
 
-from test.lib.testing import eq_, assert_raises, assert_raises_message
-from test.lib.util import gc_collect
+from sqlalchemy.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing.util import gc_collect
 import sqlalchemy as sa
-from test.lib import testing
+from sqlalchemy import testing
 from sqlalchemy import Integer, String, ForeignKey, exc as sa_exc
-from test.lib.schema import Table
-from test.lib.schema import Column
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
                         attributes, deferred, exc as orm_exc, defer, undefer,\
                         strategies, state, lazyload, backref, Session
-from test.lib import fixtures
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 
 
@@ -46,7 +46,7 @@ class ExpireTest(_fixtures.FixtureTest):
         # change the value in the DB
         users.update(users.c.id==7, values=dict(name='jack')).execute()
         sess.expire(u)
-        # object isnt refreshed yet, using dict to bypass trigger
+        # object isn't refreshed yet, using dict to bypass trigger
         assert u.__dict__.get('name') != 'jack'
         assert 'name' in attributes.instance_state(u).expired_attributes
 
@@ -67,7 +67,7 @@ class ExpireTest(_fixtures.FixtureTest):
         u = s.query(User).get(7)
         s.expunge_all()
 
-        assert_raises_message(sa_exc.InvalidRequestError, 
+        assert_raises_message(sa_exc.InvalidRequestError,
                         r"is not persistent within this Session", s.expire, u)
 
     def test_get_refreshes(self):
@@ -88,6 +88,13 @@ class ExpireTest(_fixtures.FixtureTest):
             u = s.query(User).get(10)  # expire flag reset, so not expired
         self.assert_sql_count(testing.db, go, 0)
 
+    def test_get_on_deleted_expunges(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+
         s.expire_all()
         s.execute(users.delete().where(User.id==10))
 
@@ -96,33 +103,54 @@ class ExpireTest(_fixtures.FixtureTest):
         assert s.query(User).get(10) is None
         assert u not in s # and expunges
 
-        # trick the "deleted" flag so we can re-add for the sake
-        # of this test
-        del attributes.instance_state(u).deleted
+    def test_refresh_on_deleted_raises(self):
+        users, User = self.tables.users, self.classes.User
 
-        # add it back
-        s.add(u)
-        # nope, raises ObjectDeletedError
-        assert_raises(sa.orm.exc.ObjectDeletedError, getattr, u, 'name')
+        mapper(User, users)
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+        s.expire_all()
 
-        # do a get()/remove u from session again
+        s.expire_all()
+        s.execute(users.delete().where(User.id==10))
+
+        # raises ObjectDeletedError
+        assert_raises_message(
+            sa.orm.exc.ObjectDeletedError,
+            "Instance '<User at .*?>' has been "
+            "deleted, or its row is otherwise not present.",
+            getattr, u, 'name'
+        )
+
+    def test_rollback_undoes_expunge_from_deleted(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+        s.expire_all()
+        s.execute(users.delete().where(User.id==10))
+
+        # do a get()/remove u from session
         assert s.query(User).get(10) is None
         assert u not in s
 
         s.rollback()
 
         assert u in s
-        # but now its back, rollback has occurred, the _remove_newly_deleted
-        # is reverted
+        # but now its back, rollback has occurred, the
+        # _remove_newly_deleted is reverted
         eq_(u.name, 'chuck')
 
     def test_deferred(self):
-        """test that unloaded, deferred attributes aren't included in the expiry list."""
+        """test that unloaded, deferred attributes aren't included in the
+        expiry list."""
 
         Order, orders = self.classes.Order, self.tables.orders
 
 
-        mapper(Order, orders, properties={'description':deferred(orders.c.description)})
+        mapper(Order, orders, properties={
+                    'description':deferred(orders.c.description)})
 
         s = create_session()
         o1 = s.query(Order).first()
@@ -132,6 +160,24 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'description' not in o1.__dict__
         assert o1.description
 
+    def test_deferred_notfound(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users, properties={
+            'name':deferred(users.c.name)
+        })
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+
+        assert 'name' not in u.__dict__
+        s.execute(users.delete().where(User.id==10))
+        assert_raises_message(
+            sa.orm.exc.ObjectDeletedError,
+            "Instance '<User at .*?>' has been "
+            "deleted, or its row is otherwise not present.",
+            getattr, u, 'name'
+        )
+
     def test_lazyload_autoflushes(self):
         users, Address, addresses, User = (self.tables.users,
                                 self.classes.Address,
@@ -139,28 +185,29 @@ class ExpireTest(_fixtures.FixtureTest):
                                 self.classes.User)
 
         mapper(User, users, properties={
-            'addresses':relationship(Address, order_by=addresses.c.email_address)
+            'addresses':relationship(Address,
+                    order_by=addresses.c.email_address)
         })
         mapper(Address, addresses)
         s = create_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
         adlist = u.addresses
         eq_(adlist, [
-            Address(email_address='ed@bettyboop.com'), 
+            Address(email_address='ed@bettyboop.com'),
             Address(email_address='ed@lala.com'),
-            Address(email_address='ed@wood.com'), 
+            Address(email_address='ed@wood.com'),
         ])
         a1 = u.addresses[2]
         a1.email_address = 'aaaaa'
         s.expire(u, ['addresses'])
         eq_(u.addresses, [
-            Address(email_address='aaaaa'), 
-            Address(email_address='ed@bettyboop.com'), 
+            Address(email_address='aaaaa'),
+            Address(email_address='ed@bettyboop.com'),
             Address(email_address='ed@lala.com'),
         ])
 
     def test_refresh_collection_exception(self):
-        """test graceful failure for currently unsupported 
+        """test graceful failure for currently unsupported
         immediate refresh of a collection"""
 
         users, Address, addresses, User = (self.tables.users,
@@ -175,12 +222,12 @@ class ExpireTest(_fixtures.FixtureTest):
         mapper(Address, addresses)
         s = create_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
-        assert_raises_message(sa_exc.InvalidRequestError, 
-                        "properties specified for refresh", 
+        assert_raises_message(sa_exc.InvalidRequestError,
+                        "properties specified for refresh",
                         s.refresh, u, ['addresses'])
 
         # in contrast to a regular query with no columns
-        assert_raises_message(sa_exc.InvalidRequestError, 
+        assert_raises_message(sa_exc.InvalidRequestError,
                         "no columns with which to SELECT", s.query().all)
 
     def test_refresh_cancels_expire(self):
@@ -619,7 +666,7 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'addresses' not in u.__dict__
 
         # hit the lazy loader.  just does the lazy load,
-        # doesnt do the overall refresh
+        # doesn't do the overall refresh
         def go():
             assert u.addresses[0].email_address=='ed@wood.com'
         self.assert_sql_count(testing.db, go, 1)
@@ -815,7 +862,7 @@ class ExpireTest(_fixtures.FixtureTest):
                                 self.classes.User)
 
         mapper(User, users, properties={
-            'addresses':relationship(Address, backref='user', lazy='joined', 
+            'addresses':relationship(Address, backref='user', lazy='joined',
                                     order_by=addresses.c.id),
             })
         mapper(Address, addresses)
@@ -894,7 +941,7 @@ class ExpireTest(_fixtures.FixtureTest):
         u1 = sess.query(User).options(undefer(User.name)).first()
         assert 'name' not in attributes.instance_state(u1).callables
 
-        # mass expire, the attribute was loaded, 
+        # mass expire, the attribute was loaded,
         # the attribute gets the callable
         sess.expire(u1)
         assert isinstance(
@@ -907,7 +954,7 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' not in attributes.instance_state(u1).callables
 
         # mass expire, attribute was loaded but then deleted,
-        # the callable goes away - the state wants to flip 
+        # the callable goes away - the state wants to flip
         # it back to its "deferred" loader.
         sess.expunge_all()
         u1 = sess.query(User).options(undefer(User.name)).first()
@@ -1117,7 +1164,7 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
         # which attach to u1 will expect to be "pending"
         sess.expire(u1, ['addresses'])
 
-        # attach an Address.  now its "pending" 
+        # attach an Address.  now its "pending"
         # in user.addresses
         a2 = Address(email_address='a2')
         a2.user = u1
@@ -1201,7 +1248,7 @@ class RefreshTest(_fixtures.FixtureTest):
 
     def test_refresh_with_lazy(self):
         """test that when a lazy loader is set as a trigger on an object's attribute
-        (at the attribute level, not the class level), a refresh() operation doesnt
+        (at the attribute level, not the class level), a refresh() operation doesn't
         fire the lazy loader or create any problems"""
 
         User, Address, addresses, users = (self.classes.User,
