@@ -1,13 +1,17 @@
+import datetime
 import sqlalchemy as sa
-from sqlalchemy.test import engines, testing
-from sqlalchemy import Integer, String, ForeignKey, literal_column, \
-    orm, exc, select
-from sqlalchemy.test.schema import Table, Column
+from sqlalchemy.testing import engines
+from sqlalchemy import testing
+from sqlalchemy import Integer, String, Date, ForeignKey, literal_column, \
+    orm, exc, select, TypeDecorator
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, Session, \
-    create_session, column_property, sessionmaker, clear_mappers
-from sqlalchemy.test.testing import eq_, ne_, assert_raises, assert_raises_message
-from test.orm import _base, _fixtures
-from test.engine import _base as engine_base
+    create_session, column_property, sessionmaker,\
+    exc as orm_exc
+from sqlalchemy.testing import eq_, ne_, assert_raises, assert_raises_message
+from sqlalchemy.testing import fixtures
+from test.orm import _fixtures
+from sqlalchemy.testing import fixtures
 
 
 _uuids = [
@@ -31,7 +35,7 @@ def make_uuid():
     """generate uuids even on Python 2.4 which has no 'uuid'"""
     return _uuids.pop(0)
 
-class VersioningTest(_base.MappedTest):
+class VersioningTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
@@ -43,27 +47,25 @@ class VersioningTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Foo(_base.ComparableEntity):
+        class Foo(cls.Basic):
             pass
 
+    def _fixture(self):
+        Foo, version_table = self.classes.Foo, self.tables.version_table
+
+        mapper(Foo, version_table,
+                version_id_col=version_table.c.version_id)
+        s1 = Session()
+        return s1
+
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_notsane_warning(self):
-        # clear the warning module's ignores to force the SAWarning this
-        # test relies on to be emitted (it may have already been ignored
-        # forever by other VersioningTests)
-        try:
-            del __warningregistry__
-        except NameError:
-            pass
+        Foo = self.classes.Foo
 
         save = testing.db.dialect.supports_sane_rowcount
         testing.db.dialect.supports_sane_rowcount = False
         try:
-            mapper(Foo, version_table, 
-                    version_id_col=version_table.c.version_id)
-
-            s1 = create_session(autocommit=False)
+            s1 = self._fixture()
             f1 = Foo(value='f1')
             f2 = Foo(value='f2')
             s1.add_all((f1, f2))
@@ -74,14 +76,11 @@ class VersioningTest(_base.MappedTest):
         finally:
             testing.db.dialect.supports_sane_rowcount = save
 
-    @testing.emits_warning(r'.*does not support updated rowcount')
-    @engines.close_open_connections
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support (update|delete)d rowcount')
     def test_basic(self):
-        mapper(Foo, version_table, 
-                version_id_col=version_table.c.version_id)
+        Foo = self.classes.Foo
 
-        s1 = create_session(autocommit=False)
+        s1 = self._fixture()
         f1 = Foo(value='f1')
         f2 = Foo(value='f2')
         s1.add_all((f1, f2))
@@ -100,7 +99,7 @@ class VersioningTest(_base.MappedTest):
         # Only dialects with a sane rowcount can detect the
         # StaleDataError
         if testing.db.dialect.supports_sane_rowcount:
-            assert_raises_message(sa.orm.exc.StaleDataError, 
+            assert_raises_message(sa.orm.exc.StaleDataError,
             r"UPDATE statement on table 'version_table' expected "
             r"to update 1 row\(s\); 0 were matched.",
             s1.commit),
@@ -108,7 +107,7 @@ class VersioningTest(_base.MappedTest):
         else:
             s1.commit()
 
-        # new in 0.5 !  dont need to close the session
+        # new in 0.5 !  don't need to close the session
         f1 = s1.query(Foo).get(f1.id)
         f2 = s1.query(Foo).get(f2.id)
 
@@ -120,27 +119,26 @@ class VersioningTest(_base.MappedTest):
 
         if testing.db.dialect.supports_sane_rowcount:
             assert_raises_message(
-                sa.orm.exc.StaleDataError, 
+                sa.orm.exc.StaleDataError,
                 r"DELETE statement on table 'version_table' expected "
                 r"to delete 2 row\(s\); 1 were matched.",
                 s1.commit)
         else:
             s1.commit()
 
-    @testing.emits_warning(r'.*does not support updated rowcount')
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support (update|delete)d rowcount')
     def test_bump_version(self):
         """test that version number can be bumped.
 
-        Ensures that the UPDATE or DELETE is against the 
-        last committed version of version_id_col, not the modified 
+        Ensures that the UPDATE or DELETE is against the
+        last committed version of version_id_col, not the modified
         state.
 
         """
-        mapper(Foo, version_table, 
-                version_id_col=version_table.c.version_id)
 
-        s1 = sessionmaker()()
+        Foo = self.classes.Foo
+
+        s1 = self._fixture()
         f1 = Foo(value='f1')
         s1.add(f1)
         s1.commit()
@@ -163,13 +161,13 @@ class VersioningTest(_base.MappedTest):
 
     @testing.emits_warning(r'.*does not support updated rowcount')
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_versioncheck(self):
         """query.with_lockmode performs a 'version check' on an already loaded instance"""
 
-        s1 = create_session(autocommit=False)
+        Foo = self.classes.Foo
 
-        mapper(Foo, version_table, version_id_col=version_table.c.version_id)
+
+        s1 = self._fixture()
         f1s1 = Foo(value='f1 value')
         s1.add(f1s1)
         s1.commit()
@@ -181,7 +179,7 @@ class VersioningTest(_base.MappedTest):
 
         # load, version is wrong
         assert_raises_message(
-                sa.orm.exc.StaleDataError, 
+                sa.orm.exc.StaleDataError,
                 r"Instance .* has version id '\d+' which does not "
                 r"match database-loaded version id '\d+'",
                 s1.query(Foo).with_lockmode('read').get, f1s1.id
@@ -201,13 +199,13 @@ class VersioningTest(_base.MappedTest):
     @testing.emits_warning(r'.*does not support updated rowcount')
     @engines.close_open_connections
     @testing.requires.update_nowait
-    @testing.resolve_artifact_names
     def test_versioncheck_for_update(self):
         """query.with_lockmode performs a 'version check' on an already loaded instance"""
 
-        s1 = create_session(autocommit=False)
+        Foo = self.classes.Foo
 
-        mapper(Foo, version_table, version_id_col=version_table.c.version_id)
+
+        s1 = self._fixture()
         f1s1 = Foo(value='f1 value')
         s1.add(f1s1)
         s1.commit()
@@ -229,9 +227,11 @@ class VersioningTest(_base.MappedTest):
 
     @testing.emits_warning(r'.*does not support updated rowcount')
     @engines.close_open_connections
-    @testing.resolve_artifact_names
     def test_noversioncheck(self):
         """test query.with_lockmode works when the mapper has no version id col"""
+
+        Foo, version_table = self.classes.Foo, self.tables.version_table
+
         s1 = create_session(autocommit=False)
         mapper(Foo, version_table)
         f1s1 = Foo(value="foo", version_id=0)
@@ -243,9 +243,129 @@ class VersioningTest(_base.MappedTest):
         assert f1s2.id == f1s1.id
         assert f1s2.value == f1s1.value
 
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
+    def test_merge_no_version(self):
+        Foo = self.classes.Foo
 
+        s1 = self._fixture()
+        f1 = Foo(value='f1')
+        s1.add(f1)
+        s1.commit()
 
-class RowSwitchTest(_base.MappedTest):
+        f1.value = 'f2'
+        s1.commit()
+
+        f2 = Foo(id=f1.id, value='f3')
+        f3 = s1.merge(f2)
+        assert f3 is f1
+        s1.commit()
+        eq_(f3.version_id, 3)
+
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
+    def test_merge_correct_version(self):
+        Foo = self.classes.Foo
+
+        s1 = self._fixture()
+        f1 = Foo(value='f1')
+        s1.add(f1)
+        s1.commit()
+
+        f1.value = 'f2'
+        s1.commit()
+
+        f2 = Foo(id=f1.id, value='f3', version_id=2)
+        f3 = s1.merge(f2)
+        assert f3 is f1
+        s1.commit()
+        eq_(f3.version_id, 3)
+
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
+    def test_merge_incorrect_version(self):
+        Foo = self.classes.Foo
+
+        s1 = self._fixture()
+        f1 = Foo(value='f1')
+        s1.add(f1)
+        s1.commit()
+
+        f1.value = 'f2'
+        s1.commit()
+
+        f2 = Foo(id=f1.id, value='f3', version_id=1)
+        assert_raises_message(
+            orm_exc.StaleDataError,
+            "Version id '1' on merged state "
+            "<Foo at .*?> does not match existing version '2'. "
+            "Leave the version attribute unset when "
+            "merging to update the most recent version.",
+            s1.merge, f2
+        )
+
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
+    def test_merge_incorrect_version_not_in_session(self):
+        Foo = self.classes.Foo
+
+        s1 = self._fixture()
+        f1 = Foo(value='f1')
+        s1.add(f1)
+        s1.commit()
+
+        f1.value = 'f2'
+        s1.commit()
+
+        f2 = Foo(id=f1.id, value='f3', version_id=1)
+        s1.close()
+
+        assert_raises_message(
+            orm_exc.StaleDataError,
+            "Version id '1' on merged state "
+            "<Foo at .*?> does not match existing version '2'. "
+            "Leave the version attribute unset when "
+            "merging to update the most recent version.",
+            s1.merge, f2
+        )
+
+class ColumnTypeTest(fixtures.MappedTest):
+
+    @classmethod
+    def define_tables(cls, metadata):
+        class SpecialType(TypeDecorator):
+            impl = Date
+            def process_bind_param(self, value, dialect):
+                assert isinstance(value, datetime.date)
+                return value
+
+        Table('version_table', metadata,
+              Column('id', SpecialType, primary_key=True),
+              Column('version_id', Integer, nullable=False),
+              Column('value', String(40), nullable=False))
+
+    @classmethod
+    def setup_classes(cls):
+        class Foo(cls.Basic):
+            pass
+
+    def _fixture(self):
+        Foo, version_table = self.classes.Foo, self.tables.version_table
+
+        mapper(Foo, version_table,
+                version_id_col=version_table.c.version_id)
+        s1 = Session()
+        return s1
+
+    @engines.close_open_connections
+    def test_update(self):
+        Foo = self.classes.Foo
+
+        s1 = self._fixture()
+        f1 = Foo(id=datetime.date.today(), value='f1')
+        s1.add(f1)
+        s1.commit()
+
+        f1.value='f1rev2'
+        s1.commit()
+
+class RowSwitchTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('p', metadata,
@@ -261,22 +381,28 @@ class RowSwitchTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class P(_base.ComparableEntity):
+        class P(cls.Basic):
             pass
-        class C(_base.ComparableEntity):
+        class C(cls.Basic):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
-        mapper(P, p, version_id_col=p.c.version_id, 
+        p, c, C, P = (cls.tables.p,
+                                cls.tables.c,
+                                cls.classes.C,
+                                cls.classes.P)
+
+        mapper(P, p, version_id_col=p.c.version_id,
             properties={
             'c':relationship(C, uselist=False, cascade='all, delete-orphan')
         })
         mapper(C, c, version_id_col=c.c.version_id)
 
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
     def test_row_switch(self):
+        P = self.classes.P
+
         session = sessionmaker()()
         session.add(P(id='P1', data='P version 1'))
         session.commit()
@@ -287,8 +413,10 @@ class RowSwitchTest(_base.MappedTest):
         session.add(P(id='P1', data="really a row-switch"))
         session.commit()
 
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
     def test_child_row_switch(self):
+        P, C = self.classes.P, self.classes.C
+
         assert P.c.property.strategy.use_get
 
         session = sessionmaker()()
@@ -304,7 +432,7 @@ class RowSwitchTest(_base.MappedTest):
         p.c = C(data='child row-switch')
         session.commit()
 
-class AlternateGeneratorTest(_base.MappedTest):
+class AlternateGeneratorTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('p', metadata,
@@ -320,15 +448,19 @@ class AlternateGeneratorTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class P(_base.ComparableEntity):
+        class P(cls.Basic):
             pass
-        class C(_base.ComparableEntity):
+        class C(cls.Basic):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
-        mapper(P, p, version_id_col=p.c.version_id, 
+        p, c, C, P = (cls.tables.p,
+                                cls.tables.c,
+                                cls.classes.C,
+                                cls.classes.P)
+
+        mapper(P, p, version_id_col=p.c.version_id,
             version_id_generator=lambda x:make_uuid(),
             properties={
             'c':relationship(C, uselist=False, cascade='all, delete-orphan')
@@ -337,8 +469,10 @@ class AlternateGeneratorTest(_base.MappedTest):
                     version_id_generator=lambda x:make_uuid(),
         )
 
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
     def test_row_switch(self):
+        P = self.classes.P
+
         session = sessionmaker()()
         session.add(P(id='P1', data='P version 1'))
         session.commit()
@@ -349,8 +483,10 @@ class AlternateGeneratorTest(_base.MappedTest):
         session.add(P(id='P1', data="really a row-switch"))
         session.commit()
 
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support (update|delete)d rowcount')
     def test_child_row_switch_one(self):
+        P, C = self.classes.P, self.classes.C
+
         assert P.c.property.strategy.use_get
 
         session = sessionmaker()()
@@ -366,11 +502,13 @@ class AlternateGeneratorTest(_base.MappedTest):
         p.c = C(data='child row-switch')
         session.commit()
 
-    @testing.resolve_artifact_names
+    @testing.emits_warning_on('+zxjdbc', r'.*does not support (update|delete)d rowcount')
     def test_child_row_switch_two(self):
+        P = self.classes.P
+
         Session = sessionmaker()
 
-        # TODO: not sure this test is 
+        # TODO: not sure this test is
         # testing exactly what its looking for
 
         sess1 = Session()
@@ -391,15 +529,17 @@ class AlternateGeneratorTest(_base.MappedTest):
         sess1.commit()
 
         p2.data = 'P overwritten by concurrent tx'
-        assert_raises_message(
-            orm.exc.StaleDataError,
-            r"UPDATE statement on table 'p' expected to update "
-            r"1 row\(s\); 0 were matched.",
+        if testing.db.dialect.supports_sane_rowcount:
+            assert_raises_message(
+                orm.exc.StaleDataError,
+                r"UPDATE statement on table 'p' expected to update "
+                r"1 row\(s\); 0 were matched.",
+                sess2.commit
+            )
+        else:
             sess2.commit
-        )
 
-
-class InheritanceTwoVersionIdsTest(_base.MappedTest):
+class InheritanceTwoVersionIdsTest(fixtures.MappedTest):
     """Test versioning where both parent/child table have a
     versioning column.
 
@@ -407,7 +547,7 @@ class InheritanceTwoVersionIdsTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('base', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('version_id', Integer, nullable=True),
             Column('data', String(50))
         )
@@ -419,14 +559,18 @@ class InheritanceTwoVersionIdsTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Base(_base.ComparableEntity):
+        class Base(cls.Basic):
             pass
         class Sub(Base):
             pass
 
-    @testing.resolve_artifact_names
     def test_base_both(self):
-        mapper(Base, base, 
+        Base, sub, base, Sub = (self.classes.Base,
+                                self.tables.sub,
+                                self.tables.base,
+                                self.classes.Sub)
+
+        mapper(Base, base,
                 version_id_col=base.c.version_id)
         mapper(Sub, sub, inherits=Base)
 
@@ -438,9 +582,13 @@ class InheritanceTwoVersionIdsTest(_base.MappedTest):
         # base is populated
         eq_(select([base.c.version_id]).scalar(), 1)
 
-    @testing.resolve_artifact_names
     def test_sub_both(self):
-        mapper(Base, base, 
+        Base, sub, base, Sub = (self.classes.Base,
+                                self.tables.sub,
+                                self.tables.base,
+                                self.classes.Sub)
+
+        mapper(Base, base,
                 version_id_col=base.c.version_id)
         mapper(Sub, sub, inherits=Base)
 
@@ -455,10 +603,14 @@ class InheritanceTwoVersionIdsTest(_base.MappedTest):
         # base is populated
         eq_(select([base.c.version_id]).scalar(), 1)
 
-    @testing.resolve_artifact_names
     def test_sub_only(self):
+        Base, sub, base, Sub = (self.classes.Base,
+                                self.tables.sub,
+                                self.tables.base,
+                                self.classes.Sub)
+
         mapper(Base, base)
-        mapper(Sub, sub, inherits=Base, 
+        mapper(Sub, sub, inherits=Base,
                 version_id_col=sub.c.version_id)
 
         session = Session()
@@ -472,9 +624,13 @@ class InheritanceTwoVersionIdsTest(_base.MappedTest):
         # base is not
         eq_(select([base.c.version_id]).scalar(), None)
 
-    @testing.resolve_artifact_names
     def test_mismatch_version_col_warning(self):
-        mapper(Base, base, 
+        Base, sub, base, Sub = (self.classes.Base,
+                                self.tables.sub,
+                                self.tables.base,
+                                self.classes.Sub)
+
+        mapper(Base, base,
                 version_id_col=base.c.version_id)
 
         assert_raises_message(
@@ -485,5 +641,5 @@ class InheritanceTwoVersionIdsTest(_base.MappedTest):
             "version_id_col should only be specified on "
             "the base-most mapper that includes versioning.",
             mapper,
-            Sub, sub, inherits=Base, 
+            Sub, sub, inherits=Base,
                 version_id_col=sub.c.version_id)
