@@ -1,16 +1,18 @@
-from test.lib.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing import eq_, assert_raises_message, assert_raises
 import datetime
 from sqlalchemy.schema import CreateSequence, DropSequence
-from sqlalchemy.sql import select, text, literal_column
+from sqlalchemy.sql import select, text
 import sqlalchemy as sa
-from test.lib import testing, engines
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
 from sqlalchemy import MetaData, Integer, String, ForeignKey, Boolean, exc,\
                 Sequence, func, literal, Unicode
 from sqlalchemy.types import TypeDecorator, TypeEngine
-from test.lib.schema import Table, Column
-from test.lib.testing import eq_
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.dialects import sqlite
-from test.lib import fixtures
+from sqlalchemy.testing import fixtures
+
+t = f = f2 = ts = currenttime = metadata = default_generator = None
 
 class DefaultTest(fixtures.TestBase):
 
@@ -20,7 +22,7 @@ class DefaultTest(fixtures.TestBase):
 
         db = testing.db
         metadata = MetaData(db)
-        default_generator = {'x':50}
+        default_generator = {'x': 50}
 
         def mydefault():
             default_generator['x'] += 1
@@ -132,10 +134,13 @@ class DefaultTest(fixtures.TestBase):
 
     def test_bad_arg_signature(self):
         ex_msg = \
-          "ColumnDefault Python function takes zero or one positional arguments"
+          "ColumnDefault Python function takes zero "\
+          "or one positional arguments"
 
-        def fn1(x, y): pass
-        def fn2(x, y, z=3): pass
+        def fn1(x, y):
+            pass
+        def fn2(x, y, z=3):
+            pass
         class fn3(object):
             def __init__(self, x, y):
                 pass
@@ -150,28 +155,33 @@ class DefaultTest(fixtures.TestBase):
                                      sa.ColumnDefault, fn)
 
     def test_arg_signature(self):
-        def fn1(): pass
-        def fn2(): pass
-        def fn3(x=1): pass
-        def fn4(x=1, y=2, z=3): pass
+        def fn1():
+            pass
+        def fn2():
+            pass
+        def fn3(x=1):
+            eq_(x, 1)
+        def fn4(x=1, y=2, z=3):
+            eq_(x, 1)
         fn5 = list
-        class fn6(object):
+        class fn6a(object):
             def __init__(self, x):
-                pass
-        class fn6(object):
+                eq_(x, "context")
+        class fn6b(object):
             def __init__(self, x, y=3):
-                pass
+                eq_(x, "context")
         class FN7(object):
             def __call__(self, x):
-                pass
+                eq_(x, "context")
         fn7 = FN7()
         class FN8(object):
             def __call__(self, x, y=3):
-                pass
+                eq_(x, "context")
         fn8 = FN8()
 
-        for fn in fn1, fn2, fn3, fn4, fn5, fn6, fn7, fn8:
+        for fn in fn1, fn2, fn3, fn4, fn5, fn6a, fn6b, fn7, fn8:
             c = sa.ColumnDefault(fn)
+            c.arg("context")
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_standalone(self):
@@ -279,7 +289,7 @@ class DefaultTest(fixtures.TestBase):
         eq_(set(r.context.postfetch_cols),
             set([t.c.col3, t.c.col5, t.c.col4, t.c.col6]))
 
-        eq_(t.select(t.c.col1==54).execute().fetchall(),
+        eq_(t.select(t.c.col1 == 54).execute().fetchall(),
             [(54, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, None)])
 
@@ -287,10 +297,10 @@ class DefaultTest(fixtures.TestBase):
     def test_insertmany(self):
         # MySQL-Python 1.2.2 breaks functions in execute_many :(
         if (testing.against('mysql+mysqldb') and
-            testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
+                testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
             return
 
-        r = t.insert().execute({}, {}, {})
+        t.insert().execute({}, {}, {})
 
         ctexec = currenttime.scalar()
         l = t.select().execute()
@@ -305,7 +315,7 @@ class DefaultTest(fixtures.TestBase):
 
     def test_no_embed_in_sql(self):
         """Using a DefaultGenerator, Sequence, DefaultClause
-        in the columns, where clause of a select, or in the values 
+        in the columns, where clause of a select, or in the values
         clause of insert, update, raises an informative error"""
 
         for const in (
@@ -330,16 +340,16 @@ class DefaultTest(fixtures.TestBase):
             )
 
     def test_missing_many_param(self):
-        assert_raises_message(exc.StatementError, 
+        assert_raises_message(exc.StatementError,
             "A value is required for bind parameter 'col7', in parameter group 1",
             t.insert().execute,
-            {'col4':7, 'col7':12, 'col8':19},
-            {'col4':7, 'col8':19},
-            {'col4':7, 'col7':12, 'col8':19},
+            {'col4': 7, 'col7': 12, 'col8': 19},
+            {'col4': 7, 'col8': 19},
+            {'col4': 7, 'col7': 12, 'col8': 19},
         )
 
     def test_insert_values(self):
-        t.insert(values={'col3':50}).execute()
+        t.insert(values={'col3': 50}).execute()
         l = t.select().execute()
         eq_(50, l.first()['col3'])
 
@@ -347,18 +357,18 @@ class DefaultTest(fixtures.TestBase):
     def test_updatemany(self):
         # MySQL-Python 1.2.2 breaks functions in execute_many :(
         if (testing.against('mysql+mysqldb') and
-            testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
+                testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
             return
 
         t.insert().execute({}, {}, {})
 
-        t.update(t.c.col1==sa.bindparam('pkval')).execute(
-            {'pkval':51,'col7':None, 'col8':None, 'boolcol1':False})
+        t.update(t.c.col1 == sa.bindparam('pkval')).execute(
+            {'pkval': 51, 'col7': None, 'col8': None, 'boolcol1': False})
 
-        t.update(t.c.col1==sa.bindparam('pkval')).execute(
-            {'pkval':51,},
-            {'pkval':52,},
-            {'pkval':53,})
+        t.update(t.c.col1 == sa.bindparam('pkval')).execute(
+            {'pkval': 51},
+            {'pkval': 52},
+            {'pkval': 53})
 
         l = t.select().execute()
         ctexec = currenttime.scalar()
@@ -375,9 +385,9 @@ class DefaultTest(fixtures.TestBase):
     def test_update(self):
         r = t.insert().execute()
         pk = r.inserted_primary_key[0]
-        t.update(t.c.col1==pk).execute(col4=None, col5=None)
+        t.update(t.c.col1 == pk).execute(col4=None, col5=None)
         ctexec = currenttime.scalar()
-        l = t.select(t.c.col1==pk).execute()
+        l = t.select(t.c.col1 == pk).execute()
         l = l.first()
         eq_(l,
             (pk, 'im the update', f2, None, None, ctexec, True, False,
@@ -388,8 +398,8 @@ class DefaultTest(fixtures.TestBase):
     def test_update_values(self):
         r = t.insert().execute()
         pk = r.inserted_primary_key[0]
-        t.update(t.c.col1==pk, values={'col3': 55}).execute()
-        l = t.select(t.c.col1==pk).execute()
+        t.update(t.c.col1 == pk, values={'col3': 55}).execute()
+        l = t.select(t.c.col1 == pk).execute()
         l = l.first()
         eq_(55, l['col3'])
 
@@ -420,7 +430,8 @@ class PKDefaultTest(fixtures.TablesTest):
         if not returning and not testing.db.dialect.implicit_returning:
             engine = testing.db
         else:
-            engine = engines.testing_engine(options={'implicit_returning':returning})
+            engine = engines.testing_engine(
+                            options={'implicit_returning': returning})
         engine.execute(t2.insert(), nextid=1)
         r = engine.execute(t1.insert(), data='hi')
         eq_([1], r.inserted_primary_key)
@@ -464,13 +475,13 @@ class PKIncrementTest(fixtures.TablesTest):
         self.assert_(last not in ids)
         ids.add(last)
 
-        rs = bind.execute(aitable.insert(values={'int1':func.length('four')}))
+        rs = bind.execute(aitable.insert(values={'int1': func.length('four')}))
         last = rs.inserted_primary_key[0]
         self.assert_(last)
         self.assert_(last not in ids)
         ids.add(last)
 
-        eq_(ids, set([1,2,3,4]))
+        eq_(ids, set([1, 2, 3, 4]))
 
         eq_(list(bind.execute(aitable.select().order_by(aitable.c.id))),
             [(1, 1, None), (2, None, 'row 2'), (3, 3, 'row 3'), (4, 4, None)])
@@ -499,18 +510,14 @@ class PKIncrementTest(fixtures.TablesTest):
 class EmptyInsertTest(fixtures.TestBase):
     @testing.exclude('sqlite', '<', (3, 3, 8), 'no empty insert support')
     @testing.fails_on('oracle', 'FIXME: unknown')
+    @testing.provide_metadata
     def test_empty_insert(self):
-        metadata = MetaData(testing.db)
-        t1 = Table('t1', metadata,
+        t1 = Table('t1', self.metadata,
                 Column('is_true', Boolean, server_default=('1')))
-        metadata.create_all()
-
-        try:
-            result = t1.insert().execute()
-            eq_(1, select([func.count(text('*'))], from_obj=t1).scalar())
-            eq_(True, t1.select().scalar())
-        finally:
-            metadata.drop_all()
+        self.metadata.create_all()
+        t1.insert().execute()
+        eq_(1, select([func.count(text('*'))], from_obj=t1).scalar())
+        eq_(True, t1.select().scalar())
 
 class AutoIncrementTest(fixtures.TablesTest):
     __requires__ = ('identity',)
@@ -552,6 +559,30 @@ class AutoIncrementTest(fixtures.TablesTest):
         )
         assert t._autoincrement_column is None
 
+    def test_autoincrement_ignore_fk(self):
+        m = MetaData()
+        Table('y', m,
+            Column('id', Integer(), primary_key=True)
+        )
+        x = Table('x', m,
+            Column('id', Integer(),
+                ForeignKey('y.id'),
+                autoincrement="ignore_fk", primary_key=True)
+        )
+        assert x._autoincrement_column is x.c.id
+
+    def test_autoincrement_fk_disqualifies(self):
+        m = MetaData()
+        Table('y', m,
+            Column('id', Integer(), primary_key=True)
+        )
+        x = Table('x', m,
+            Column('id', Integer(),
+                ForeignKey('y.id'),
+                primary_key=True)
+        )
+        assert x._autoincrement_column is None
+
     @testing.fails_on('sqlite', 'FIXME: unknown')
     def test_non_autoincrement(self):
         # sqlite INT primary keys can be non-unique! (only for ints)
@@ -560,18 +591,45 @@ class AutoIncrementTest(fixtures.TablesTest):
             Column('data', String(20)))
         nonai.create()
 
-
-        try:
+        def go():
             # postgresql + mysql strict will fail on first row,
             # mysql in legacy mode fails on second row
             nonai.insert().execute(data='row 1')
             nonai.insert().execute(data='row 2')
-            assert False
-        except sa.exc.DBAPIError, e:
-            assert True
+        assert_raises(
+            sa.exc.DBAPIError,
+            go
+        )
 
         nonai.insert().execute(id=1, data='row 1')
 
+
+    def test_col_w_sequence_non_autoinc_no_firing(self):
+        metadata = self.metadata
+        # plain autoincrement/PK table in the actual schema
+        Table("x", metadata,
+            Column("set_id", Integer, primary_key=True)
+        )
+        metadata.create_all()
+
+        # for the INSERT use a table with a Sequence
+        # and autoincrement=False.  Using a ForeignKey
+        # would have the same effect
+        dataset_no_autoinc = Table("x", MetaData(),
+            Column("set_id", Integer, Sequence("some_seq"),
+                        primary_key=True, autoincrement=False)
+            )
+
+        testing.db.execute(
+                dataset_no_autoinc.insert()
+        )
+        eq_(
+            testing.db.scalar(dataset_no_autoinc.count()), 1
+        )
+
+
+
+
 class SequenceDDLTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     __dialect__ = 'default'
 
@@ -606,7 +664,7 @@ class SequenceExecTest(fixtures.TestBase):
 
     @classmethod
     def setup_class(cls):
-        cls.seq= Sequence("my_sequence")
+        cls.seq = Sequence("my_sequence")
         cls.seq.create(testing.db)
 
     @classmethod
@@ -628,7 +686,7 @@ class SequenceExecTest(fixtures.TestBase):
         self._assert_seq_result(s.execute(testing.db))
 
     def test_explicit_optional(self):
-        """test dialect executes a Sequence, returns nextval, whether 
+        """test dialect executes a Sequence, returns nextval, whether
         or not "optional" is set """
 
         s = Sequence("my_sequence", optional=True)
@@ -669,7 +727,7 @@ class SequenceExecTest(fixtures.TestBase):
             Column('x', Integer)
         )
         t1.create(testing.db)
-        testing.db.execute(t1.insert(), [{'x':1}, {'x':300}, {'x':301}])
+        testing.db.execute(t1.insert(), [{'x': 1}, {'x': 300}, {'x': 301}])
         s = Sequence("my_sequence")
         eq_(
             testing.db.execute(
@@ -697,11 +755,11 @@ class SequenceExecTest(fixtures.TestBase):
 
     @testing.provide_metadata
     def test_inserted_pk_no_returning(self):
-        """test inserted_primary_key contains [None] when 
+        """test inserted_primary_key contains [None] when
         pk_col=next_value(), implicit returning is not used."""
 
         metadata = self.metadata
-        e = engines.testing_engine(options={'implicit_returning':False})
+        e = engines.testing_engine(options={'implicit_returning': False})
         s = Sequence("my_sequence")
         metadata.bind = e
         t1 = Table('t', metadata,
@@ -716,11 +774,11 @@ class SequenceExecTest(fixtures.TestBase):
     @testing.requires.returning
     @testing.provide_metadata
     def test_inserted_pk_implicit_returning(self):
-        """test inserted_primary_key contains the result when 
+        """test inserted_primary_key contains the result when
         pk_col=next_value(), when implicit returning is used."""
 
         metadata = self.metadata
-        e = engines.testing_engine(options={'implicit_returning':True})
+        e = engines.testing_engine(options={'implicit_returning': True})
         s = Sequence("my_sequence")
         metadata.bind = e
         t1 = Table('t', metadata,
@@ -738,8 +796,8 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     @testing.fails_on('firebird', 'no FB support for start/increment')
     def test_start_increment(self):
         for seq in (
-                Sequence('foo_seq'), 
-                Sequence('foo_seq', start=8), 
+                Sequence('foo_seq'),
+                Sequence('foo_seq', start=8),
                 Sequence('foo_seq', increment=5)):
             seq.create(testing.db)
             try:
@@ -758,21 +816,21 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         return testing.db.dialect.has_sequence(testing.db, name)
 
     def test_nextval_render(self):
-        """test dialect renders the "nextval" construct, 
+        """test dialect renders the "nextval" construct,
         whether or not "optional" is set """
 
         for s in (
-                Sequence("my_seq"), 
+                Sequence("my_seq"),
                 Sequence("my_seq", optional=True)):
             assert str(s.next_value().
                     compile(dialect=testing.db.dialect)) in (
-                "nextval('my_seq')",
-                "gen_id(my_seq, 1)",
-                "my_seq.nextval",
-            )
+                        "nextval('my_seq')",
+                        "gen_id(my_seq, 1)",
+                        "my_seq.nextval",
+                    )
 
     def test_nextval_unsupported(self):
-        """test next_value() used on non-sequence platform 
+        """test next_value() used on non-sequence platform
         raises NotImplementedError."""
 
         s = Sequence("my_seq")
@@ -795,7 +853,7 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 
     def test_checkfirst_metadata(self):
         m = MetaData()
-        s = Sequence("my_sequence", metadata=m)
+        Sequence("my_sequence", metadata=m)
         m.create_all(testing.db, checkfirst=False)
         assert self._has_sequence('my_sequence')
         m.create_all(testing.db, checkfirst=True)
@@ -817,15 +875,15 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     @testing.provide_metadata
     def test_table_overrides_metadata_create(self):
         metadata = self.metadata
-        s1 = Sequence("s1", metadata=metadata)
+        Sequence("s1", metadata=metadata)
         s2 = Sequence("s2", metadata=metadata)
         s3 = Sequence("s3")
-        t = Table('t', metadata, 
+        t = Table('t', metadata,
                     Column('c', Integer, s3, primary_key=True))
         assert s3.metadata is metadata
 
 
-        t.create(testing.db)
+        t.create(testing.db, checkfirst=True)
         s3.drop(testing.db)
 
         # 't' is created, and 's3' won't be
@@ -845,6 +903,7 @@ class SequenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         assert not self._has_sequence('s2')
 
 
+cartitems = sometable = metadata = None
 class TableBoundSequenceTest(fixtures.TestBase):
     __requires__ = ('sequences',)
 
@@ -857,10 +916,10 @@ class TableBoundSequenceTest(fixtures.TestBase):
             Column("description", String(40)),
             Column("createdate", sa.DateTime())
         )
-        sometable = Table( 'Manager', metadata,
-               Column('obj_id', Integer, Sequence('obj_id_seq'), ),
-               Column('name', String(128)),
-               Column('id', Integer, Sequence('Manager_id_seq', optional=True),
+        sometable = Table('Manager', metadata,
+                Column('obj_id', Integer, Sequence('obj_id_seq')),
+                Column('name', String(128)),
+                Column('id', Integer, Sequence('Manager_id_seq', optional=True),
                       primary_key=True),
            )
 
@@ -888,7 +947,8 @@ class TableBoundSequenceTest(fixtures.TestBase):
     def test_seq_nonpk(self):
         """test sequences fire off as defaults on non-pk columns"""
 
-        engine = engines.testing_engine(options={'implicit_returning':False})
+        engine = engines.testing_engine(
+                            options={'implicit_returning': False})
         result = engine.execute(sometable.insert(), name="somename")
 
         assert set(result.postfetch_cols()) == set([sometable.c.obj_id])
@@ -897,8 +957,8 @@ class TableBoundSequenceTest(fixtures.TestBase):
         assert set(result.postfetch_cols()) == set([sometable.c.obj_id])
 
         sometable.insert().execute(
-            {'name':'name3'},
-            {'name':'name4'})
+            {'name': 'name3'},
+            {'name': 'name4'})
         eq_(sometable.select().order_by(sometable.c.id).execute().fetchall(),
             [(1, "somename", 1),
              (2, "someother", 2),
@@ -947,7 +1007,9 @@ class SpecialTypePKTest(fixtures.TestBase):
         r = t.insert().values(data=5).execute()
 
         # we don't pre-fetch 'server_default'.
-        if 'server_default' in kw and (not testing.db.dialect.implicit_returning or not implicit_returning):
+        if 'server_default' in kw and (not
+                    testing.db.dialect.implicit_returning or
+                    not implicit_returning):
             eq_(r.inserted_primary_key, [None])
         else:
             eq_(r.inserted_primary_key, ['INT_1'])
@@ -993,7 +1055,7 @@ class SpecialTypePKTest(fixtures.TestBase):
 class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_string_default_none_on_insert(self):
-        """Test that without implicit returning, we return None for 
+        """Test that without implicit returning, we return None for
         a string server default.
 
         That is, we don't want to attempt to pre-execute "server_default"
@@ -1003,7 +1065,7 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
         """
 
         metadata = self.metadata
-        t = Table('x', metadata, 
+        t = Table('x', metadata,
                 Column('y', String(10), server_default='key_one', primary_key=True),
                 Column('data', String(10)),
                 implicit_returning=False
@@ -1022,7 +1084,7 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
         """With implicit_returning, we get a string PK default back no problem."""
 
         metadata = self.metadata
-        t = Table('x', metadata, 
+        t = Table('x', metadata,
                 Column('y', String(10), server_default='key_one', primary_key=True),
                 Column('data', String(10))
                 )
@@ -1037,8 +1099,8 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_int_default_none_on_insert(self):
         metadata = self.metadata
-        t = Table('x', metadata, 
-                Column('y', Integer, 
+        t = Table('x', metadata,
+                Column('y', Integer,
                         server_default='5', primary_key=True),
                 Column('data', String(10)),
                 implicit_returning=False
@@ -1060,8 +1122,8 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_autoincrement_reflected_from_server_default(self):
         metadata = self.metadata
-        t = Table('x', metadata, 
-                Column('y', Integer, 
+        t = Table('x', metadata,
+                Column('y', Integer,
                         server_default='5', primary_key=True),
                 Column('data', String(10)),
                 implicit_returning=False
@@ -1076,8 +1138,8 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_int_default_none_on_insert_reflected(self):
         metadata = self.metadata
-        t = Table('x', metadata, 
-                Column('y', Integer, 
+        Table('x', metadata,
+                Column('y', Integer,
                         server_default='5', primary_key=True),
                 Column('data', String(10)),
                 implicit_returning=False
@@ -1104,8 +1166,8 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_int_default_on_insert_with_returning(self):
         metadata = self.metadata
-        t = Table('x', metadata, 
-                Column('y', Integer, 
+        t = Table('x', metadata,
+                Column('y', Integer,
                         server_default='5', primary_key=True),
                 Column('data', String(10))
                 )
@@ -1120,7 +1182,7 @@ class ServerDefaultsOnPKTest(fixtures.TestBase):
 
 class UnicodeDefaultsTest(fixtures.TestBase):
     def test_no_default(self):
-        c = Column(Unicode(32))
+        Column(Unicode(32))
 
     def test_unicode_default(self):
         # Py3K
@@ -1128,7 +1190,7 @@ class UnicodeDefaultsTest(fixtures.TestBase):
         # Py2K
         default = u'foo'
         # end Py2K
-        c = Column(Unicode(32), default=default)
+        Column(Unicode(32), default=default)
 
 
     def test_nonunicode_default(self):
