@@ -2,7 +2,7 @@
 """verrrrry basic unicode column name testing"""
 
 from sqlalchemy import *
-from sqlalchemy.testing import fixtures, engines
+from sqlalchemy.testing import fixtures, engines, eq_
 from sqlalchemy import testing, util
 from sqlalchemy.testing.engines import utf8_engine
 from sqlalchemy.sql import column
@@ -86,6 +86,29 @@ class UnicodeSchemaTest(fixtures.TestBase):
         assert t2.select().execute().fetchall() == [(1, 1)]
         assert t3.select().execute().fetchall() == [(1, 5, 1, 1)]
 
+    def test_col_targeting(self):
+        t1.insert().execute({u'méil': 1, u'\u6e2c\u8a66': 5})
+        t2.insert().execute({u'a': 1, u'b': 1})
+        t3.insert().execute({u'\u6e2c\u8a66_id': 1,
+                             u'unitable1_\u6e2c\u8a66': 5,
+                             u'Unitéble2_b': 1,
+                             u'\u6e2c\u8a66_self': 1})
+
+        row = t1.select().execute().first()
+        eq_(row[t1.c[u'méil']], 1)
+        eq_(row[t1.c[u'\u6e2c\u8a66']], 5)
+
+        row = t2.select().execute().first()
+        eq_(row[t2.c[u'a']], 1)
+        eq_(row[t2.c[u'b']], 1)
+
+        row = t3.select().execute().first()
+        eq_(row[t3.c[u'\u6e2c\u8a66_id']], 1)
+        eq_(row[t3.c[u'unitable1_\u6e2c\u8a66']], 5)
+        eq_(row[t3.c[u'Unitéble2_b']], 1)
+        eq_(row[t3.c[u'\u6e2c\u8a66_self']], 1)
+
+
     @testing.skip_if(lambda: util.pypy,
             "pypy/sqlite3 reports unicode cursor.description "
             "incorrectly pre 2.2, workaround applied in 0.9")
@@ -119,6 +142,19 @@ class UnicodeSchemaTest(fixtures.TestBase):
         meta.drop_all()
         metadata.create_all()
 
+    def test_repr(self):
+
+        m = MetaData()
+        t = Table(u'\u6e2c\u8a66', m, Column(u'\u6e2c\u8a66_id', Integer))
+
+        if util.py2k:
+            eq_(
+                repr(t),
+                (
+                    "Table(u'\\u6e2c\\u8a66', MetaData(bind=None), "
+                    "Column(u'\\u6e2c\\u8a66_id', Integer(), table=<\\u6e2c\\u8a66>), "
+                    "schema=None)"))
+
 class EscapesDefaultsTest(fixtures.TestBase):
     def test_default_exec(self):
         metadata = MetaData(testing.db)
